import logging
import uuid

from django.db import transaction
from django.db.models import PositiveIntegerField, CheckConstraint, Q, F
from django.forms import model_to_dict

from isc_common import setAttr, delAttr
from isc_common.bit import TurnBitOn
from isc_common.fields.code_field import CodeField
from isc_common.fields.name_field import NameField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditModel, AuditQuerySet
from isc_common.models.tree_audit import TreeAuditModelManager
from isc_common.number import StrToNumber
from isc_common.progress import managed_progress, ProgressDroped
from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.item_refs import Item_refs
from kaf_pas.kd.models.document_attributes import Document_attributes, Document_attributesManager

logger = logging.getLogger(__name__)


class Item_lineQuerySet(AuditQuerySet):

    def _check(self, **kwargs):
        if kwargs.get('parent') != None:
            if kwargs.get('parent' == kwargs.get('child')):
                raise Exception(f'Attempt to write circular reference')

    def _checkDefaults(self, defaults):
        if defaults != None:
            if (defaults.get("SPC_CLM_MARK_id") == None and defaults.get("SPC_CLM_NAME_id") == None) and (defaults.get("SPC_CLM_MARK") == None and defaults.get("SPC_CLM_NAME") == None):
                raise Exception(f'SPC_CLM_MARK and SPC_CLM_NAME not been Null together. ({defaults})')

    def create(self, **kwargs):
        self._check(**kwargs)
        section = kwargs.get('section')
        section_num = Item_lineManager.section_num(section)
        setAttr(kwargs, 'section_num', section_num)
        return super().create(**kwargs)

    def get_or_create(self, defaults=None, **kwargs):
        self._check(**kwargs)
        self._checkDefaults(defaults)
        section = kwargs.get('section')
        if section:
            section_num = Item_lineManager.section_num(section)
            setAttr(kwargs, 'section_num', section_num)
        elif isinstance(defaults, dict):
            section = defaults.get('section')
            section_num = Item_lineManager.section_num(section)
            setAttr(defaults, 'section_num', section_num)

        return super().get_or_create(defaults, **kwargs)

    def update(self, **kwargs):
        self._check(**kwargs)
        section = kwargs.get('section')
        section_num = Item_lineManager.section_num(section)
        setAttr(kwargs, 'section_num', section_num)
        delAttr(kwargs, 'confirmed')
        return super().update(**kwargs)

    def update_or_create(self, defaults=None, **kwargs):
        self._check(**kwargs)
        section = kwargs.get('section')
        self._checkDefaults(defaults)
        if section:
            section_num = Item_lineManager.section_num(section)
            setAttr(kwargs, 'section_num', section_num)
        elif isinstance(defaults, dict):
            section = defaults.get('section')
            section_num = Item_lineManager.section_num(section)
            setAttr(defaults, 'section_num', section_num)
        return super().get_or_create(defaults, **kwargs)


class Item_lineManager(TreeAuditModelManager):
    def get_queryset(self):
        return Item_lineQuerySet(self.model, using=self._db)

    @staticmethod
    def section_num(section):
        sorted_section = {"Документация": 0, "Комплексы": 1, "Сборочные единицы": 2, "Детали": 3, "Стандартные изделия": 4, "Прочие изделия": 5, "Материалы": 6, "Комплекты": 7}
        return sorted_section.get(section)

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'child_id': record.child.id if record.child else None,
            'parent_id': record.parent.id,
            'SPC_CLM_FORMAT_id': record.SPC_CLM_FORMAT.id if record.SPC_CLM_FORMAT else None,
            'SPC_CLM_FORMAT__value_str': record.SPC_CLM_FORMAT.value_str if record.SPC_CLM_FORMAT else None,
            'SPC_CLM_ZONE_id': record.SPC_CLM_ZONE.id if record.SPC_CLM_ZONE else None,
            'SPC_CLM_ZONE__value_str': record.SPC_CLM_ZONE.value_str if record.SPC_CLM_ZONE else None,
            'SPC_CLM_POS_id': record.SPC_CLM_POS.id if record.SPC_CLM_POS else None,
            'SPC_CLM_POS__value_str': record.SPC_CLM_POS.value_str if record.SPC_CLM_POS else None,
            'SPC_CLM_POS__value_int': record.SPC_CLM_POS.value_int if record.SPC_CLM_POS else None,
            'SPC_CLM_MARK_id': record.SPC_CLM_MARK.id if record.SPC_CLM_MARK else None,
            'SPC_CLM_MARK__value_str': record.SPC_CLM_MARK.value_str if record.SPC_CLM_MARK else None,
            'SPC_CLM_NAME_id': record.SPC_CLM_NAME.id if record.SPC_CLM_NAME else None,
            'SPC_CLM_NAME__value_str': record.SPC_CLM_NAME.value_str if record.SPC_CLM_NAME else None,
            'SPC_CLM_COUNT_id': record.SPC_CLM_COUNT.id if record.SPC_CLM_COUNT else None,
            'SPC_CLM_COUNT__value_str': record.SPC_CLM_COUNT.value_str if record.SPC_CLM_COUNT else None,
            'SPC_CLM_NOTE_id': record.SPC_CLM_NOTE.id if record.SPC_CLM_NOTE else None,
            'SPC_CLM_NOTE__value_str': record.SPC_CLM_NOTE.value_str if record.SPC_CLM_NOTE else None,
            'SPC_CLM_MASSA_id': record.SPC_CLM_MASSA.id if record.SPC_CLM_MASSA else None,
            'SPC_CLM_MASSA__value_str': record.SPC_CLM_MASSA.value_str if record.SPC_CLM_MASSA else None,
            'SPC_CLM_MATERIAL_id': record.SPC_CLM_MATERIAL.id if record.SPC_CLM_MATERIAL else None,
            'SPC_CLM_MATERIAL__value_str': record.SPC_CLM_MATERIAL.value_str if record.SPC_CLM_MATERIAL else None,
            'SPC_CLM_USER_id': record.SPC_CLM_USER.id if record.SPC_CLM_USER else None,
            'SPC_CLM_USER__value_str': record.SPC_CLM_USER.value_str if record.SPC_CLM_USER else None,
            'SPC_CLM_KOD_id': record.SPC_CLM_KOD.id if record.SPC_CLM_KOD else None,
            'SPC_CLM_KOD__value_str': record.SPC_CLM_KOD.value_str if record.SPC_CLM_KOD else None,
            'SPC_CLM_FACTORY_id': record.SPC_CLM_FACTORY.id if record.SPC_CLM_FACTORY else None,
            'SPC_CLM_FACTORY__value_str': record.SPC_CLM_FACTORY.value_str if record.SPC_CLM_FACTORY else None,
            'section': record.section,
            'section_num': record.section_num,
            'subsection': record.subsection,
            'lastmodified': record.lastmodified,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    @staticmethod
    def getIcon(record):
        if record.isFolder:
            return 'folder_256.png'
        elif record.section != None:
            _section = record.section.lower()
            if _section == 'документация':
                return 'documentation.png'
            elif _section == 'комплексы':
                return 'complexes.png'
            elif _section == 'сборочные единицы':
                return 'assembly.png'
            elif _section == 'детали':
                return 'part.png'
            elif _section == 'стандартные изделия':
                return 'standard_prod.png'
            elif _section == 'прочие изделия':
                return 'other.png'
            elif _section == 'материалы':
                return 'materials.png'
            elif _section == 'комплекты':
                return 'kits.png'
            elif _section == 'ведомость':
                return 'templates.png'
            else:
                return 'question.png'
        else:
            return 'question.png'

    @property
    def item_name(self):
        if self.SPC_CLM_NAME != None and self.SPC_CLM_MARK != None:
            return f'{self.SPC_CLM_NAME.value_str}: {self.SPC_CLM_MARK.value_str}'
        elif self.SPC_CLM_NAME == None and self.SPC_CLM_MARK != None:
            return self.SPC_CLM_MARK.value_str
        elif self.SPC_CLM_NAME != None and self.SPC_CLM_MARK == None:
            return self.SPC_CLM_NAME.value_str
        else:
            return 'Неизвестен'

    def createFromRequest(self, request, removed=None, function=None):
        request = DSRequest(request=request)
        data = request.get_data()

        parent_id = data.get('parent_id')
        child_id = data.get('child_id')

        if parent_id == None:
            raise Exception(f'parent_id is None')

        if child_id == None:
            raise Exception(f'child_id is None')

        with transaction.atomic():

            _data = dict(
                parent_id=parent_id,
                child_id=child_id,

                SPC_CLM_FORMAT_id=data.get('SPC_CLM_FORMAT_id'),
                SPC_CLM_ZONE_id=data.get('SPC_CLM_ZONE_id'),
                SPC_CLM_POS_id=data.get('SPC_CLM_POS_id'),
                SPC_CLM_MARK_id=data.get('SPC_CLM_MARK_id'),
                SPC_CLM_NAME_id=data.get('SPC_CLM_NAME_id'),
                SPC_CLM_COUNT_id=data.get('SPC_CLM_COUNT_id'),
                SPC_CLM_NOTE_id=data.get('SPC_CLM_NOTE_id'),
                SPC_CLM_MASSA_id=data.get('SPC_CLM_MASSA_id'),
                SPC_CLM_MATERIAL_id=data.get('SPC_CLM_MATERIAL_id'),
                SPC_CLM_USER_id=data.get('SPC_CLM_USER_id'),
                SPC_CLM_KOD_id=data.get('SPC_CLM_KOD_id'),
                SPC_CLM_FACTORY_id=data.get('SPC_CLM_FACTORY_id'),
                section=data.get('section'),
                subsection=data.get('subsection'),
            )

            if data.get('SPC_CLM_MARK_id') == None and data.get('SPC_CLM_NAME_id') == None:
                raise Exception(f'SPC_CLM_MARK and SPC_CLM_NAME not been Null together. ({self.model})')

            Item_refs.objects.get_or_create(child_id=child_id, parent_id=parent_id)

            res = super().create(**_data)
            setAttr(data, 'id', res.id)
        return data

    def updateFromRequest(self, request, removed=None, function=None):
        request = DSRequest(request=request)
        id = request.get_id()
        data = request.get_data()
        data_res = request.get_data()
        mode = data.get('mode')

        if mode == 'item_spw_replace':
            dropRecords = data.get('dropRecords')

            if not dropRecords:
                raise Exception(f'Не выбран источник замены.')

            targetRecord = data.get('targetRecord')

            if not targetRecord:
                raise Exception(f'Не выбрана цель замены.')

            dropRecord = dropRecords[0]

            with transaction.atomic():
                targetLine = Item_line.objects.get(child_id=targetRecord.get('child_id'), parent_id=targetRecord.get('parent_id'))
                dropItem = Item.objects.get(id=dropRecord.get('id'))

                try:
                    Item_refs.objects.get(child=targetLine.child, parent_id=targetLine.parent).delete()
                except Item_refs.DoesNotExist:
                    pass

                targetLine.child_id = dropItem.id
                targetLine.SPC_CLM_NAME = dropItem.STMP_1
                setAttr(targetRecord, 'child_id', dropItem.id)
                setAttr(targetRecord, 'SPC_CLM_NAME_id', dropItem.STMP_1.id if dropItem.STMP_1 else None)
                setAttr(targetRecord, 'SPC_CLM_NAME__value_str', dropItem.STMP_1.value_str if dropItem.STMP_1 else None)
                setAttr(targetRecord, 'SPC_CLM_MARK_id', dropItem.STMP_2.id if dropItem.STMP_2 else None)
                setAttr(targetRecord, 'SPC_CLM_MARK__value_str', dropItem.STMP_2.value_str if dropItem.STMP_2 else None)
                targetLine.SPC_CLM_MARK = dropItem.STMP_2
                targetLine.save()

                Item_refs.objects.get_or_create(child=targetLine.child, parent=targetLine.parent)

                return targetRecord

        relevant = data.get('relevant')
        confirmed = data.get('confirmed')
        item_props = data.get('item_props')
        child_id = data.get('child_id')

        if relevant == 'Актуален':
            item_props |= Item.props.relevant
        elif relevant == 'Не актуален':
            item_props &= ~Item.props.relevant

        if confirmed == 'Подтвержден':
            item_props |= Item.props.confirmed
        elif confirmed == 'Не подтвержден':
            item_props &= ~Item.props.confirmed

        delAttr(data, 'document_id')
        delAttr(data, 'isFolder')
        delAttr(data, 'icon')
        delAttr(data, 'groupParentId')
        delAttr(data, 'where_from')
        delAttr(data, 'relevant')
        delAttr(data, 'confirmed')
        delAttr(data, 'line_id')
        delAttr(data, 'item_props')
        delAttr(data, 'mode')
        _data = dict()

        for key, val in data.items():
            if str(key).find('__value_str') == -1 and str(key).find('__value_int') == -1:
                setAttr(_data, key, val)

        with transaction.atomic():
            res = super().filter(id=id).update(**_data)
            Item.objects.update_or_create(id=child_id, defaults=dict(props=item_props))
        return data_res

    def copyFromRequest(self, request, removed=None, ):
        from kaf_pas.ckk.models.item_line_view import Item_line_view
        from isc_common.models.deleted_progresses import Deleted_progresses

        request = DSRequest(request=request)

        data = request.json.get('data')
        with managed_progress(
                qty=len(data.get('records')),
                id=str(uuid.uuid4()),
                user=request.user,
                title="Выполнено",
                props=TurnBitOn(0, 0)
        ) as progress:
            try:
                with transaction.atomic():
                    progress.setContentsLabel(f'<b>Создание строк детализации</b>')
                    for record in data.get('records'):
                        for item_line in Item_line_view.objects.filter(
                                SPC_CLM_NAME__value_str=record.get('STMP_1__value_str'),
                                SPC_CLM_MARK__value_str=record.get('STMP_2__value_str')
                        ):
                            res, _ = Item_line.objects.get_or_create(
                                parent_id=record.get("parent_id"),
                                child_id=record.get("id"),
                                SPC_CLM_FORMAT=item_line.SPC_CLM_FORMAT,
                                SPC_CLM_ZONE=item_line.SPC_CLM_ZONE,
                                SPC_CLM_POS=item_line.SPC_CLM_POS,
                                SPC_CLM_MARK=item_line.SPC_CLM_MARK,
                                SPC_CLM_NAME=item_line.SPC_CLM_NAME,
                                SPC_CLM_COUNT=item_line.SPC_CLM_COUNT,
                                SPC_CLM_NOTE=item_line.SPC_CLM_NOTE,
                                SPC_CLM_MASSA=item_line.SPC_CLM_MASSA,
                                SPC_CLM_MATERIAL=item_line.SPC_CLM_MATERIAL,
                                SPC_CLM_USER=item_line.SPC_CLM_USER,
                                SPC_CLM_KOD=item_line.SPC_CLM_USER,
                                SPC_CLM_FACTORY=item_line.SPC_CLM_FACTORY,
                                section=item_line.section,
                                section_num=item_line.section_num,
                                subsection=item_line.subsection,
                            )
                            break
                        progress.step()
            except ProgressDroped as ex:
                Deleted_progresses.objects.filter(id_progress=progress.id, user=progress.user).delete()
                raise ex

    def lookupFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()

        attr_type__code = data.get('attr_type__code')
        value_str = data.get(f'{attr_type__code}__value_str')
        value_int = data.get(f'{attr_type__code}__value_int')

        if value_str == None and value_int != None:
            value_str = str(value_int)

        if value_str == None:
            return None

        res, created = Document_attributesManager.get_or_create_attribute(attr_codes=attr_type__code, value_str=value_str)
        res = model_to_dict(res)
        if res.get('value_str') == 'null':
            setAttr(res, 'value_str', None)
        return res

    def deleteFromRequest(self, request, removed=None, ):
        from kaf_pas.ckk.models.item_refs import Item_refs
        request = DSRequest(request=request)
        res = 0
        with transaction.atomic():
            for id, mode in request.get_tuple_ids():
                if mode == 'hide':
                    ...
                else:
                    for item_line in super().filter(id=id).select_for_update():
                        Item_refs.objects1.filter(parent=item_line.parent, child=item_line.child).delete_with_checked()
                        item_line.delete()
                        res += 1
        return res


class Item_line(AuditModel):
    parent = ForeignKeyProtect(Item, verbose_name='Товарная позиция', related_name='item_parent')
    child = ForeignKeyProtect(Item, verbose_name='Товарная позиция', related_name='item_child')

    SPC_CLM_FORMAT = ForeignKeyProtect(Document_attributes, verbose_name='Формат', related_name='SPC_CLM_FORMAT', null=True, blank=True)
    SPC_CLM_ZONE = ForeignKeyProtect(Document_attributes, verbose_name='Зона', related_name='SPC_CLM_ZONE', null=True, blank=True)
    SPC_CLM_POS = ForeignKeyProtect(Document_attributes, verbose_name='Позиция', related_name='SPC_CLM_POS', null=True, blank=True)
    SPC_CLM_MARK = ForeignKeyProtect(Document_attributes, verbose_name='Обозначение', related_name='SPC_CLM_MARK', null=True, blank=True)
    SPC_CLM_NAME = ForeignKeyProtect(Document_attributes, verbose_name='Наименование', related_name='SPC_CLM_NAME', null=True, blank=True)
    SPC_CLM_COUNT = ForeignKeyProtect(Document_attributes, verbose_name='Количество', related_name='SPC_CLM_COUNT', null=True, blank=True)
    SPC_CLM_NOTE = ForeignKeyProtect(Document_attributes, verbose_name='Примечание', related_name='SPC_CLM_NOTE', null=True, blank=True)
    SPC_CLM_MASSA = ForeignKeyProtect(Document_attributes, verbose_name='Масса', related_name='SPC_CLM_MASSA', null=True, blank=True)
    SPC_CLM_MATERIAL = ForeignKeyProtect(Document_attributes, verbose_name='Материал', related_name='SPC_CLM_MATERIAL', null=True, blank=True)
    SPC_CLM_USER = ForeignKeyProtect(Document_attributes, verbose_name='Пользовательская', related_name='SPC_CLM_USER', null=True, blank=True)
    SPC_CLM_KOD = ForeignKeyProtect(Document_attributes, verbose_name='Код', related_name='SPC_CLM_KOD', null=True, blank=True)
    SPC_CLM_FACTORY = ForeignKeyProtect(Document_attributes, verbose_name='Предприятие - изготовитель', related_name='SPC_CLM_FACTORY', null=True, blank=True)
    section = CodeField()
    section_num = PositiveIntegerField(null=True, blank=True)
    subsection = NameField()

    Документ_на_материал = ForeignKeyProtect(Document_attributes, related_name='Документ_на_материал_Item_line', null=True, blank=True)
    Наименование_материала = ForeignKeyProtect(Document_attributes, related_name='Наименование_материала_Item_line', null=True, blank=True)
    Документ_на_сортамент = ForeignKeyProtect(Document_attributes, related_name='Документ_на_сортамент_Item_line', null=True, blank=True)
    Наименование_сортамента = ForeignKeyProtect(Document_attributes, related_name='Наименование_сортамента_Item_line', null=True, blank=True)

    objects = Item_lineManager()

    @property
    def qty(self):
        if self.SPC_CLM_COUNT == None:
            return None

        if self.SPC_CLM_COUNT.value_int == None:
            return StrToNumber(self.SPC_CLM_COUNT.value_str)
        return self.SPC_CLM_COUNT.value_int

    def __str__(self):
        return f'\n\nchild=[{self.child}]\n' \
               f'\nparent=[{self.parent}]\n' \
               f'\nSPC_CLM_FORMAT=[{self.SPC_CLM_FORMAT}]' \
               f'\nSPC_CLM_ZONE=[{self.SPC_CLM_ZONE}]' \
               f'\nSPC_CLM_POS=[{self.SPC_CLM_POS}]' \
               f'\nSPC_CLM_MARK=[{self.SPC_CLM_MARK}]' \
               f'\nSPC_CLM_NAME=[{self.SPC_CLM_NAME}]' \
               f'\nSPC_CLM_COUNT=[{self.SPC_CLM_COUNT}]' \
               f'\nSPC_CLM_NOTE=[{self.SPC_CLM_NOTE}]' \
               f'\nSPC_CLM_MASSA=[{self.SPC_CLM_MASSA}]' \
               f'\nSPC_CLM_MATERIAL=[{self.SPC_CLM_MATERIAL}]' \
               f'\nSPC_CLM_USER=[{self.SPC_CLM_USER}]' \
               f'\nSPC_CLM_KOD=[{self.SPC_CLM_KOD}' \
               f'\nSPC_CLM_FACTORY=[{self.SPC_CLM_FACTORY}]\n' \
               f'section={self.section}\n' \
               f'subsection={self.subsection}\n\n'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Строка состава изделия'
        unique_together = (('parent', 'child'),)
        constraints = [CheckConstraint(check=~Q(child=F('parent')), name='Item_line_not_circulate_refs')]
        ordering = ['section']



# Generated by Django 2.2.2 on 2019-06-26 09:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0110_auto_20190624_1357'),
    ]

    operations = [
        migrations.CreateModel(
            name='Item_line_view',
            fields=[
            ],
            options={
                'verbose_name': 'Строка состава изделия',
                'db_table': 'ckk_item_line_view',
                'managed': False,
                'proxy': True,
            },
            bases=('ckk.item_line',),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-10 09:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0045_auto_20190610_0109'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='document_attr_cross',
            unique_together={('document', 'position_in_document')},
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-20 12:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('input', '0008_auto_20190820_1235'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user_add_info',
            name='ip',
            field=models.GenericIPAddressField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-29 05:32

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0114_auto_20200228_1240'),
    ]

    operations = [
        migrations.AddField(
            model_name='operation_level',
            name='props',
            field=bitfield.models.BitField((('isProductionOrder', 'Уровень заказа на производство'),), db_index=True, default=0),
        ),
    ]



import logging

from bitfield import BitField

from isc_common.bit import IsBitOn
from isc_common.models.base_ref import BaseRef, BaseRefQuerySet, StatusBaseRefManager

logger = logging.getLogger(__name__)


class Status_demandQuerySet(BaseRefQuerySet):
    pass


class Status_demandManager(StatusBaseRefManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'description': record.description,
            'props': record.props._value,
            'disabled': IsBitOn(record.props, 0),
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Status_demandQuerySet(self.model, using=self._db)


class Status_demand(BaseRef):
    props = BitField(flags=(
        ('disabled', 'Неактивная запись в гриде')
    ), default=0, db_index=True)

    objects = Status_demandManager()

    def __str__(self):
        return f"ID:{self.id}, code: {self.code}, name: {self.name}, description: {self.description}"

    class Meta:
        verbose_name = 'Статусы заказов на продажу'



from django.urls import path

from kaf_pas.ckk.views import item_refs

urlpatterns = [

    path('Item_refs/Fetch/', item_refs.Item_refs_Fetch),
    path('Item_refs/Add', item_refs.Item_refs_Add),
    path('Item_refs/Update', item_refs.Item_refs_Update),
    path('Item_refs/Remove', item_refs.Item_refs_Remove),
    path('Item_refs/Lookup/', item_refs.Item_refs_Lookup),
    path('Item_refs/Info/', item_refs.Item_refs_Info),

]



# Generated by Django 2.2 on 2019-04-14 16:16

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0017_auto_20190414_0923'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Item_line_refs',
            new_name='Item_refs',
        ),
        migrations.AlterModelOptions(
            name='item_refs',
            options={'verbose_name': 'Item_refs'},
        ),
        migrations.AlterField(
            model_name='item_line',
            name='isFolder',
            field=models.BooleanField(default=False, verbose_name='Признак наличия потомков'),
        ),
        migrations.AlterField(
            model_name='item_refs',
            name='child',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='child', to='ckk.Item'),
        ),
        migrations.AlterField(
            model_name='item_refs',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='parent', to='ckk.Item'),
        ),
    ]



import logging

from django.core.management import BaseCommand
from django.db import transaction, connection

from isc_common.logger.Logger import Logger

logger = logging.getLogger(__name__)
logger.__class__ = Logger


class Command(BaseCommand):
    help = 'Создание товаоных позиций после импорта из Лоцмана'

    def handle(self, *args, **options):
        try:
            from kaf_pas.kd.models.documents_thumb import Documents_thumbManager
            with transaction.atomic():
                with connection.cursor() as cursor:
                    cursor.execute(f'''SELECT count(*),
                                           attr_type_id,
                                           value_str
                                    FROM kd_document_attributes
                                    where value_str is not null
                                    group by attr_type_id,
                                             value_str
                                    having count(*) > 1''')
                    rows = cursor.fetchall()
                    no = 1
                    for row in rows:
                        count, attr_type_id, value_str = row
                        cursor.execute(f'''select
                                                id
                                            from
                                                kd_document_attributes
                                            where
                                                attr_type_id = %s
                                                and value_str = %s''', [attr_type_id, value_str])
                        rows1 = cursor.fetchall()
                        first_step = True
                        id1 = None
                        id2 = None
                        for row1 in rows1:
                            if not first_step:
                                id2, = row1
                                cursor.execute(f'''update kd_document_attr_cross set attribute_id=%s where attribute_id=%s''', [id1, id2])
                                cursor.execute(f'''update ckk_item set "STMP_1_id"=%s where "STMP_1_id"=%s''', [id1, id2])
                                cursor.execute(f'''update ckk_item set "STMP_2_id"=%s where "STMP_2_id"=%s''', [id1, id2])
                                cursor.execute(f'''update ckk_item_line set "SPC_CLM_COUNT_id"=%s where "SPC_CLM_COUNT_id"=%s''', [id1, id2])
                                cursor.execute(f'''update ckk_item_line set "SPC_CLM_FACTORY_id"=%s where "SPC_CLM_FACTORY_id"=%s''', [id1, id2])
                                cursor.execute(f'''update ckk_item_line set "SPC_CLM_FORMAT_id"=%s where "SPC_CLM_FORMAT_id"=%s''', [id1, id2])
                                cursor.execute(f'''update ckk_item_line set "SPC_CLM_KOD_id"=%s where "SPC_CLM_KOD_id"=%s''', [id1, id2])
                                cursor.execute(f'''update ckk_item_line set "SPC_CLM_MASSA_id"=%s where "SPC_CLM_MASSA_id"=%s''', [id1, id2])
                                cursor.execute(f'''update ckk_item_line set "SPC_CLM_MATERIAL_id"=%s where "SPC_CLM_MATERIAL_id"=%s''', [id1, id2])
                                cursor.execute(f'''update ckk_item_line set "SPC_CLM_NOTE_id"=%s where "SPC_CLM_NOTE_id"=%s''', [id1, id2])
                                cursor.execute(f'''update ckk_item_line set "SPC_CLM_POS_id"=%s where "SPC_CLM_POS_id"=%s''', [id1, id2])
                                cursor.execute(f'''update ckk_item_line set "SPC_CLM_USER_id"=%s where "SPC_CLM_USER_id"=%s''', [id1, id2])
                                cursor.execute(f'''update ckk_item_line set "SPC_CLM_ZONE_id"=%s where "SPC_CLM_ZONE_id"=%s''', [id1, id2])
                                cursor.execute(f'''update ckk_item_line set "SPC_CLM_NAME_id"=%s where "SPC_CLM_NAME_id"=%s''', [id1, id2])
                                cursor.execute(f'''update ckk_item_line set "SPC_CLM_MARK_id"=%s where "SPC_CLM_MARK_id"=%s''', [id1, id2])
                                cursor.execute(f'''update ckk_item_line set "Документ_на_материал_id"=%s where "Документ_на_материал_id"=%s''', [id1, id2])
                                cursor.execute(f'''update kd_lotsman_document_attr_cross set attribute_id=%s where attribute_id=%s''', [id1, id2])
                                cursor.execute(f'''delete from kd_document_attributes where id =%s''', [id2])
                                print(f'Deleted: #{no} {id2}')
                                no += 1
                            else:
                                id1, = row1
                                first_step = False
        except Exception as ex:
            raise ex



# Generated by Django 2.2.6 on 2019-10-10 15:00

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0152_remove_status_code_owner'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'), ('from_cdw', 'Получено из чертежа'), ('from_spw', 'Получено из спецификации'), ('for_line', 'Строка спецификации'), ('from_pdf', 'Получено из бумажного архива'), ('man_input', 'Занесено вручную'), ('copied', 'Скопировано')), db_index=True, default=1),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-10 01:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0043_auto_20190610_0056'),
    ]

    operations = [
        migrations.AddField(
            model_name='document_attr_cross',
            name='position_in_document',
            field=models.PositiveIntegerField(default=None),
        ),
    ]



import logging

from bitfield import BitField
from django.db import transaction

from isc_common import setAttr
from isc_common.auth.models.user import User
from isc_common.fields.related import ForeignKeyCascade
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditManager, AuditModel, AuditQuerySet
from isc_common.number import DelProps
from kaf_pas.production.models.resource import Resource

logger = logging.getLogger(__name__)


class Resource_usersQuerySet(AuditQuerySet):
    pass


class Resource_usersManager(AuditManager):

    def createFromRequest(self, request, removed=None):
        request = DSRequest(request=request)
        data = request.get_data()
        user_ids = data.get('user_ids', None)
        context_ids = data.get('context_ids', None)
        _data = []

        with transaction.atomic():
            if user_ids and context_ids:
                if not isinstance(user_ids, list):
                    user_ids = [user_ids]

                if not isinstance(context_ids, list):
                    context_ids = [context_ids]

                for user_id in user_ids:
                    for context_id in context_ids:
                        res = super().create(user_id=user_id, resource_id=context_id)

            return data

    def updateFromRequest(self, request, removed=None, function=None):
        if not isinstance(request, DSRequest):
            request = DSRequest(request=request)
        data = request.get_data()

        with transaction.atomic():
            props = data.get('props', 0)
            if data.get('compulsory_reading') == True:
                props |= Resource_users.props.compulsory_reading
            else:
                props &= ~Resource_users.props.compulsory_reading

            _data = dict()
            setAttr(_data, 'props', props)

            super().filter(id=data.get('id')).update(**_data)

            return data

    def deleteFromRequest(self, request, removed=None, ):
        request = DSRequest(request=request)
        res = 0
        tuple_ids = request.get_tuple_ids()
        with transaction.atomic():
            for id, mode in tuple_ids:
                if mode == 'hide':
                    super().filter(id=id).soft_delete()
                else:
                    res = super().get(id=id)
                    res.delete()
                    qty = 1
                res += qty

            return res

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'user_id': record.user.id,
            'user__username': record.user.username,
            'user__first_name': record.user.first_name,
            'user__last_name': record.user.last_name,
            'user__email': record.user.email,
            'user__middle_name': record.user.middle_name,
            'compulsory_reading': Resource_users.props.compulsory_reading,
            'props': record.props,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return DelProps(res)

    def get_queryset(self):
        return Resource_usersQuerySet(self.model, using=self._db)


class Resource_users(AuditModel):
    resource = ForeignKeyCascade(Resource)
    user = ForeignKeyCascade(User)
    props = BitField(flags=(
        ('compulsory_reading', 'Обязательное прочтение'),  # 1
    ), default=1, db_index=True)

    objects = Resource_usersManager()

    def __str__(self):
        return f"ID:{self.id}"

    class Meta:
        verbose_name = 'Ответственные пользователи ресурсов'
        unique_together = (('resource', 'user'),)



import logging

from events.events_manager import EventsManager

logger = logging.getLogger(__name__)


class EventStack:
    eventsManager = EventsManager()

    def __init__(self):
        self.EVENTS_TEST = self.eventsManager.get_event('events.test', 'События.Test', compulsory_reading=True)

        self.EVENTS_PRODUCTION_READY_2_LAUNCH = self.eventsManager.get_event('events.production.ready2Launch', 'События.Производство.Выполнен расчет готовности к запуску', compulsory_reading=False)

        self.EVENTS_PRODUCTION_MAKE_ROUTING = self.eventsManager.get_event('events.planing.route.make', 'События.Планирование.Маршрутизация.Формирование', compulsory_reading=False)
        self.EVENTS_PRODUCTION_DELETE_ROUTING = self.eventsManager.get_event('events.planing.route.delete', 'События.Планирование.Маршрутизация.Удаление', compulsory_reading=False)

        self.EVENTS_PRODUCTION_MAKE_LAUNCH = self.eventsManager.get_event('events.production.launch.make', 'События.Производство.Запуск.Формирование', compulsory_reading=False)
        self.EVENTS_PRODUCTION_DELETE_LAUNCH = self.eventsManager.get_event('events.production.launch.delete', 'События.Производство.Запуск.Удаление', compulsory_reading=False)

        self.EVENTS_PRODUCTION_ORDER_CREATE = self.eventsManager.get_event('events.production.order.create', 'События.Производство.Распоряжение на производство.Формирование', compulsory_reading=False)
        self.EVENTS_PRODUCTION_ORDER_DELETE = self.eventsManager.get_event('events.production.order.delete', 'События.Производство.Распоряжение на производство.Удаление', compulsory_reading=False)




from django.urls import path

from kaf_pas.kd.views import document_attributes_SPC_CLM_POS

urlpatterns = [

    path('Document_attributes_SPC_CLM_POS/Fetch/', document_attributes_SPC_CLM_POS.Document_attributes_SPC_CLM_POS_Fetch),
    path('Document_attributes_SPC_CLM_POS/Add', document_attributes_SPC_CLM_POS.Document_attributes_SPC_CLM_POS_Add),
    path('Document_attributes_SPC_CLM_POS/Update', document_attributes_SPC_CLM_POS.Document_attributes_SPC_CLM_POS_Update),
    path('Document_attributes_SPC_CLM_POS/Remove', document_attributes_SPC_CLM_POS.Document_attributes_SPC_CLM_POS_Remove),
    path('Document_attributes_SPC_CLM_POS/Lookup/', document_attributes_SPC_CLM_POS.Document_attributes_SPC_CLM_POS_Lookup),
    path('Document_attributes_SPC_CLM_POS/Info/', document_attributes_SPC_CLM_POS.Document_attributes_SPC_CLM_POS_Info),
    path('Document_attributes_SPC_CLM_POS/Copy', document_attributes_SPC_CLM_POS.Document_attributes_SPC_CLM_POS_Copy),

]



# Generated by Django 2.2.8 on 2019-12-14 10:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0100_lotsman_document_attr_cross_lotsman_documents_hierarcy'),
    ]

    operations = [
        migrations.AlterField(
            model_name='lotsman_documents_hierarcy',
            name='id',
            field=models.BigIntegerField(primary_key=True, serialize=False),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-22 08:37

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0018_auto_20190522_0748'),
    ]

    operations = [
        migrations.AddField(
            model_name='precent',
            name='document',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='document', to='sales.Precent'),
        ),
        migrations.AlterField(
            model_name='precent',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='parent', to='sales.Precent'),
        ),
    ]



import logging
import os

from django.conf import settings
from django.db import transaction, connection
from django.db.models import TextField, UniqueConstraint, Q

from crypto.models.crypto_file import CryptoManager, Crypto_file
from isc_common.fields.related import ForeignKeyProtect
from kaf_pas.kd.models.documents import Documents
from kaf_pas.kd.models.documents_thumb10 import Documents_thumb10
from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy

logger = logging.getLogger(__name__)


class Documents_thumbManager(CryptoManager):
    @staticmethod
    def refreshFromDbls(doc_type, table_name):
        settings.LOCKS.acquire('Documents_thumbManager.refreshFromDbls')
        with connection.cursor() as cursor:
            cursor.execute(f'''select
                                    count (*),
                                    {doc_type}_id,
                                    path
                                from
                                    kd_documents_{table_name}
                                where {doc_type}_id is not null        
                                group by
                                    {doc_type}_id,
                                    path
                                having
                                    count(*) >1
                                    ''')
            rows = cursor.fetchall()
            for row in rows:
                count, document, path = row
                cursor.execute(f'''select
                                        id
                                    from
                                        kd_documents_{table_name}
                                    where
                                        {doc_type}_id = %s
                                        and path = %s''', [document, path])
                rows1 = cursor.fetchall()
                first_step = True
                for row1 in rows1:
                    id, = row1
                    if not first_step:
                        cursor.execute(f'''delete from ckk_item_image_refs where {table_name}_id =%s''', [id])
                        cursor.execute(f'''delete
                                                from
                                                    kd_documents_{table_name}
                                                where
                                                    id = %s''', [id])
                        print(f'Deleted: {id}')
                    else:
                        first_step = False
        settings.LOCKS.release('Documents_thumbManager.refreshFromDbls')

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "path": record.file_name,
            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "deliting": record.deliting,
            "file_document_thumb_url": f'/logic/DocumentsThumb/Download/{record.id}/',

        }
        return res

    def deleteFromRequest(self, request, removed=None, ):
        res = 0
        ids = request.GET.getlist('ids')

        with transaction.atomic():
            for i in range(0, len(ids), 2):
                id = ids[i]
                mode = ids[i + 1]

                if mode == 'hide':
                    for item in super().filter(id=id):
                        item.soft_delete()
                        Documents_thumb10.objects.filter(path=item.path).soft_delete()
                        res += 1
                else:
                    for item in super().filter(id=id):
                        item.delete()
                        Documents_thumb10.objects.filter(path=item.path).delete()
                        res += 1
        return res


class Documents_thumb(Crypto_file):
    # Менять на cascade нельзя, потому как не происходит удаленеи файлов изображений при удалении документа
    document = ForeignKeyProtect(Documents, verbose_name='КД', null=True, blank=True)
    lotsman_document = ForeignKeyProtect(Lotsman_documents_hierarcy, verbose_name='Лоцман', null=True, blank=True)
    path = TextField()

    @property
    def file_name(self):
        dir, filename = os.path.split(self.path)
        return filename

    objects = Documents_thumbManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'JPEG варианты документов уменьшенная копия'
        constraints = [
            UniqueConstraint(fields=['path'], condition=Q(document=None) & Q(lotsman_document=None), name='Documents_thumb_unique_constraint_0'),
            UniqueConstraint(fields=['lotsman_document', 'path'], condition=Q(document=None), name='Documents_thumb_unique_constraint_1'),
            UniqueConstraint(fields=['document', 'path'], condition=Q(lotsman_document=None), name='Documents_thumb_unique_constraint_2'),
            UniqueConstraint(fields=['document', 'lotsman_document', 'path'], name='Documents_thumb_unique_constraint_3'),
        ]



from django.urls import path

from kaf_pas.kd.views import pathes

urlpatterns = [

    path('Pathes/Fetch/', pathes.Pathes_Fetch),
    path('Pathes/Add', pathes.Pathes_Add),
    path('Pathes/Update', pathes.Pathes_Update),
    path('Pathes/Remove', pathes.Pathes_Remove),
    path('Pathes/Lookup/', pathes.Pathes_Lookup),
    path('Pathes/Info/', pathes.Pathes_Info),

]



from django.urls import path

from kaf_pas.kd.views import document_attributes_SPC_CLM_MATERIAL

urlpatterns = [

    path('Document_attributes_SPC_CLM_MATERIAL/Fetch/', document_attributes_SPC_CLM_MATERIAL.Document_attributes_SPC_CLM_MATERIAL_Fetch),
    path('Document_attributes_SPC_CLM_MATERIAL/Add', document_attributes_SPC_CLM_MATERIAL.Document_attributes_SPC_CLM_MATERIAL_Add),
    path('Document_attributes_SPC_CLM_MATERIAL/Update', document_attributes_SPC_CLM_MATERIAL.Document_attributes_SPC_CLM_MATERIAL_Update),
    path('Document_attributes_SPC_CLM_MATERIAL/Remove', document_attributes_SPC_CLM_MATERIAL.Document_attributes_SPC_CLM_MATERIAL_Remove),
    path('Document_attributes_SPC_CLM_MATERIAL/Lookup/', document_attributes_SPC_CLM_MATERIAL.Document_attributes_SPC_CLM_MATERIAL_Lookup),
    path('Document_attributes_SPC_CLM_MATERIAL/Info/', document_attributes_SPC_CLM_MATERIAL.Document_attributes_SPC_CLM_MATERIAL_Info),
    path('Document_attributes_SPC_CLM_MATERIAL/Copy', document_attributes_SPC_CLM_MATERIAL.Document_attributes_SPC_CLM_MATERIAL_Copy),

]



# Generated by Django 3.0.2 on 2020-01-22 07:52

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0034_standard_colors_color'),
        ('accounting', '0009_remove_tmp_buffer_color'),
    ]

    operations = [
        migrations.AddField(
            model_name='tmp_buffer',
            name='color',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='isc_common.Standard_colors'),
        ),
    ]



# Generated by Django 2.2.5 on 2019-09-17 19:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0053_auto_20190917_1855'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Demand',
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-24 10:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0024_auto_20190522_1751'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='materials',
            name='parent',
        ),
    ]



from django.urls import path

from kaf_pas.production.views import status_launch

urlpatterns = [

    path('Status_launch/Fetch/', status_launch.Status_launch_Fetch),
    path('Status_launch/Add', status_launch.Status_launch_Add),
    path('Status_launch/Update', status_launch.Status_launch_Update),
    path('Status_launch/Remove', status_launch.Status_launch_Remove),
    path('Status_launch/Lookup/', status_launch.Status_launch_Lookup),
    path('Status_launch/Info/', status_launch.Status_launch_Info),
    path('Status_launch/Copy', status_launch.Status_launch_Copy),

]



# Generated by Django 2.2.8 on 2019-12-12 20:39

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0171_item_refs_syntetic_parent'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_refs',
            name='syntetic_child',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='syntetic_child', to='ckk.Item', unique=True),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-28 22:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0027_delete_operation_plan_template_locations'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operation_plan_templates',
            name='parent',
        ),
        migrations.RemoveField(
            model_name='operation_plan_templates_item_lines',
            name='item_line',
        ),
        migrations.RemoveField(
            model_name='operation_plan_templates_item_lines',
            name='operation_plan_template_item',
        ),
        migrations.DeleteModel(
            name='Operation_plan_template_items',
        ),
        migrations.DeleteModel(
            name='Operation_plan_templates',
        ),
        migrations.DeleteModel(
            name='Operation_plan_templates_item_lines',
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-17 17:58

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0096_auto_20191117_1758'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_item_line',
            name='launch',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='production.Launches'),
        ),
        migrations.AlterField(
            model_name='launch_item_refs',
            name='launch',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='production.Launches'),
        ),
    ]



import logging

from django.db.models import PositiveIntegerField, TextField

from isc_common.fields.related import ForeignKeyCascade, ForeignKeyProtect
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel
from kaf_pas.ckk.models.ed_izm import Ed_izm
from kaf_pas.production.models.operations import Operations
from kaf_pas.production.models.operations_template import Operations_template

logger = logging.getLogger(__name__)


class Operations_template_detailQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Operations_template_detailManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'operation_id': record.operation.id,
            'operation__code': record.operation.code,
            'operation__name': record.operation.name,
            'operation__full_name': record.operation.full_name,
            'operation__description': record.operation.description,
            'ed_izm_id': record.ed_izm.id if record.ed_izm else None,
            'ed_izm__code': record.ed_izm.code if record.ed_izm else None,
            'ed_izm__name': record.ed_izm.name if record.ed_izm else None,
            'ed_izm__description': record.ed_izm.description if record.ed_izm else None,
            'qty': record.qty,
            'num': record.num,
            'description': record.description,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Operations_template_detailQuerySet(self.model, using=self._db)


class Operations_template_detail(AuditModel):
    operation = ForeignKeyCascade(Operations)
    num = PositiveIntegerField(default=None, null=True, blank=True)
    qty = PositiveIntegerField(default=None, null=True, blank=True)
    description = TextField(default=None, null=True, blank=True)
    ed_izm = ForeignKeyProtect(Ed_izm, default=None, null=True, blank=True)
    template = ForeignKeyCascade(Operations_template)

    objects = Operations_template_detailManager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Детализация шаблона группы оперций'
        unique_together = (('operation', 'template'),)



# Generated by Django 2.2.1 on 2019-05-23 17:04

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0055_auto_20190523_1125'),
        ('sales', '0024_remove_precent_operation_plan_templates'),
    ]

    operations = [
        migrations.CreateModel(
            name='Precent_items',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('item', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Item')),
                ('precent', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='sales.Precent')),
            ],
            options={
                'verbose_name': 'Кросс таблица',
                'unique_together': {('item', 'precent')},
            },
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-06 12:49

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0025_auto_20191206_1151'),
        ('production', '0151_operations_planing_operation'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operations',
            name='planing_operation',
        ),
        migrations.AddField(
            model_name='operations',
            name='planing_operation_type',
            field=isc_common.fields.related.ForeignKeyCascade(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='planing_operation_type', to='planing.Operation_types'),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-20 18:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0054_operations_view_resource_view'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operation_material',
            name='material',
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-22 11:31

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0195_remove_launches_facets'),
    ]

    operations = [
        migrations.AddField(
            model_name='operations',
            name='props',
            field=bitfield.models.BitField((('assemle', 'Операция комплектации'),), db_index=True, default=0),
        ),
    ]



import logging
import sys

from django.conf import settings
from django.db import transaction

from isc_common import setAttr
from isc_common.http.DSRequest import DSRequest
from kaf_pas.kd.models.document_attributes import Document_attributes, Document_attributesManager, Document_attributesQuerySet
from kaf_pas.kd.models.document_attributes_view import Document_attributes_view, Document_attributes_viewManager

logger = logging.getLogger(__name__)


class Spw_attrsQuerySet(Document_attributesQuerySet):
    def __str__(self):
        return self.query

    def filter(self, *args, **kwargs):
        setAttr(kwargs, 'attr_type__code__contains', 'SPC')
        return super().filter(*args, **kwargs)

    @staticmethod
    def make_specification(queryResult):
        settings.LOCKS.acquire('Spw_attrsQuerySet.make_specification')
        res = []
        attr_typee_id = sys.maxsize
        id = None
        r = None
        max_position_in_raw_document = None

        index = queryResult.count()
        for record in queryResult:
            if attr_typee_id > record.attr_type.id:
                if r:
                    setAttr(r, 'id', id)
                    setAttr(r, 'max_position_in_raw_document', max_position_in_raw_document)
                    res.append(r.copy())
                id = None
                r = dict()
            attr_typee_id = record.attr_type.id
            setAttr(r, f'{record.attr_type.code}_ID', record.id)
            setAttr(r, record.attr_type.code, record.value_str)
            setAttr(r, 'document_id', record.document.id)
            setAttr(r, 'section', record.section)
            setAttr(r, 'subsection', record.subsection)
            setAttr(r, f'{record.attr_type.code}_position_in_document', record.position_in_document)
            max_position_in_raw_document = record.position_in_document
            setAttr(r, 'editing', True)
            setAttr(r, 'deliting', True)
            if id == None:
                id = str(record.id)
            else:
                id += f'${record.id}'
            index -= 1
            if index == 0:
                setAttr(r, 'id', id)
                res.append(r.copy())

        # for item in res:
        #     logger.debug(item)
        settings.LOCKS.release('Spw_attrsQuerySet.make_specification')
        return res

    def get_range_rows1(self, request, function=None):
        request = DSRequest(request=request)
        self.alive_only = request.alive_only
        criteria = self.get_criteria(json=request.json)
        queryResult = self.filter(*[], criteria).order_by(*['position_in_document'])
        res = Spw_attrsQuerySet.make_specification(queryResult)
        return res


class Spw_attrsManager(Document_attributes_viewManager):

    @staticmethod
    def rec_document_attr(document_id, attr_type_id, code, attr_type__code, value_str, position, section, subsection):
        from kaf_pas.kd.models.document_attr_cross import Document_attr_cross
        from kaf_pas.kd.models.documents import Documents

        settings.LOCKS.acquire('Spw_attrsManager.rec_document_attr')
        if attr_type_id == None:
            if code != None and attr_type__code != None:
                attr_type, _ = Document_attributesManager.get_or_create_attribute(attr_codes=code, value_str=attr_type__code)
                attr_type_id = attr_type.id

        if attr_type_id != None:
            value_int = None
            try:
                value_int = int(value_str)
            except:
                pass

            document = Documents.objects.get(id=document_id)
            document_attribute_old = Document_attributes.objects.get(id=attr_type_id)

            document_attr_cross_old = None
            try:
                document_attr_cross_old = Document_attr_cross.objects.get(
                    document=document,
                    attribute=document_attribute_old,
                    position_in_document=position
                )
            except Document_attr_cross.DoesNotExist:
                pass

            document_attribute, ctreated = Document_attributes.objects.get_or_create(
                attr_type=document_attribute_old.attr_type,
                value_str=value_str,
                value_int=value_int
            )

            position_in_document = None
            if document_attr_cross_old != None:
                position_in_document = document_attr_cross_old.position_in_document
                document_attr_cross_old.delete()

            res, created = Document_attr_cross.objects.get_or_create(
                attribute=document_attribute,
                document=document,
                position_in_document=position_in_document,
                section=section,
                subsection=subsection,
            )
            return res
        settings.LOCKS.release('Spw_attrsManager.rec_document_attr')

    @staticmethod
    def update_rec_spesification(data):
        settings.LOCKS.acquire('Spw_attrsManager.update_rec_spesification')
        with transaction.atomic():
            attr_type__code = 'SPC_CLM_FORMAT'
            Spw_attrsManager.rec_document_attr(
                document_id=data.get('document_id'),
                code=attr_type__code,
                attr_type__code=data.get(attr_type__code),
                attr_type_id=data.get(f'{attr_type__code}_ID'),
                value_str=data.get(attr_type__code),
                position=data.get(f'{attr_type__code}_position_in_document'),
                section=data.get('section'),
                subsection=data.get('subsection'),
            )

            attr_type__code = 'SPC_CLM_ZONE'
            Spw_attrsManager.rec_document_attr(
                document_id=data.get('document_id'),
                code=attr_type__code,
                attr_type__code=data.get(attr_type__code),
                attr_type_id=data.get(f'{attr_type__code}_ID'),
                value_str=data.get(attr_type__code),
                position=data.get(f'{attr_type__code}_position_in_document'),
                section=data.get('section'),
                subsection=data.get('subsection'),
            )

            attr_type__code = 'SPC_CLM_POS'
            Spw_attrsManager.rec_document_attr(
                document_id=data.get('document_id'),
                code=attr_type__code,
                attr_type__code=data.get(attr_type__code),
                attr_type_id=data.get(f'{attr_type__code}_ID'),
                value_str=data.get(attr_type__code),
                position=data.get(f'{attr_type__code}_position_in_document'),
                section=data.get('section'),
                subsection=data.get('subsection'),
            )

            attr_type__code = 'SPC_CLM_MARK'
            Spw_attrsManager.rec_document_attr(
                document_id=data.get('document_id'),
                code=attr_type__code,
                attr_type__code=data.get(attr_type__code),
                attr_type_id=data.get(f'{attr_type__code}_ID'),
                value_str=data.get(attr_type__code),
                position=data.get(f'{attr_type__code}_position_in_document'),
                section=data.get('section'),
                subsection=data.get('subsection'),
            )

            attr_type__code = 'SPC_CLM_NAME'
            Spw_attrsManager.rec_document_attr(
                document_id=data.get('document_id'),
                code=attr_type__code,
                attr_type__code=data.get(attr_type__code),
                attr_type_id=data.get(f'{attr_type__code}_ID'),
                value_str=data.get(attr_type__code),
                position=data.get(f'{attr_type__code}_position_in_document'),
                section=data.get('section'),
                subsection=data.get('subsection'),
            )

            attr_type__code = 'SPC_CLM_COUNT'
            Spw_attrsManager.rec_document_attr(
                document_id=data.get('document_id'),
                code=attr_type__code,
                attr_type__code=data.get(attr_type__code),
                attr_type_id=data.get(f'{attr_type__code}_ID'),
                value_str=data.get(attr_type__code),
                position=data.get(f'{attr_type__code}_position_in_document'),
                section=data.get('section'),
                subsection=data.get('subsection'),
            )

            attr_type__code = 'SPC_CLM_NOTE'
            Spw_attrsManager.rec_document_attr(
                document_id=data.get('document_id'),
                code=attr_type__code,
                attr_type__code=data.get(attr_type__code),
                attr_type_id=data.get(f'{attr_type__code}_ID'),
                value_str=data.get(attr_type__code),
                position=data.get(f'{attr_type__code}_position_in_document'),
                section=data.get('section'),
                subsection=data.get('subsection'),
            )

            attr_type__code = 'SPC_CLM_MASSA'
            Spw_attrsManager.rec_document_attr(
                document_id=data.get('document_id'),
                code=attr_type__code,
                attr_type__code=data.get(attr_type__code),
                attr_type_id=data.get(f'{attr_type__code}_ID'),
                value_str=data.get(attr_type__code),
                position=data.get(f'{attr_type__code}_position_in_document'),
                section=data.get('section'),
                subsection=data.get('subsection'),
            )

            attr_type__code = 'SPC_CLM_MATERIAL'
            Spw_attrsManager.rec_document_attr(
                document_id=data.get('document_id'),
                code=attr_type__code,
                attr_type__code=data.get(attr_type__code),
                attr_type_id=data.get(f'{attr_type__code}_ID'),
                value_str=data.get(attr_type__code),
                position=data.get(f'{attr_type__code}_position_in_document'),
                section=data.get('section'),
                subsection=data.get('subsection'),
            )

            attr_type__code = 'SPC_CLM_USER'
            Spw_attrsManager.rec_document_attr(
                document_id=data.get('document_id'),
                code=attr_type__code,
                attr_type__code=data.get(attr_type__code),
                attr_type_id=data.get(f'{attr_type__code}_ID'),
                value_str=data.get(attr_type__code),
                position=data.get(f'{attr_type__code}_position_in_document'),
                section=data.get('section'),
                subsection=data.get('subsection'),
            )

            attr_type__code = 'SPC_CLM_KOD'
            Spw_attrsManager.rec_document_attr(
                document_id=data.get('document_id'),
                code=attr_type__code,
                attr_type__code=data.get(attr_type__code),
                attr_type_id=data.get(f'{attr_type__code}_ID'),
                value_str=data.get(attr_type__code),
                position=data.get(f'{attr_type__code}_position_in_document'),
                section=data.get('section'),
                subsection=data.get('subsection'),
            )

            attr_type__code = 'SPC_CLM_FACTORY'
            Spw_attrsManager.rec_document_attr(
                document_id=data.get('document_id'),
                code=attr_type__code,
                attr_type__code=data.get(attr_type__code),
                attr_type_id=data.get(f'{attr_type__code}_ID'),
                value_str=data.get(attr_type__code),
                position=data.get(f'{attr_type__code}_position_in_document'),
                section=data.get('section'),
                subsection=data.get('subsection'),
            )
        settings.LOCKS.release('Spw_attrsManager.update_rec_spesification')

    def updateFromRequest(self, request, removed=None, function=None):
        request = DSRequest(request=request)
        data = request.get_data()
        Spw_attrsManager.update_rec_spesification(data)
        return data

    def get_queryset(self):
        return Spw_attrsQuerySet(self.model, using=self._db)

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "document_id": record.document.id,
            "attr_type_id": record.attr_type.id,
            "attr_type__code": record.attr_type.code,
            "attr_type__name": record.attr_type.name,
            "attr_type__description": record.attr_type.description if record.attr_type else None,
            "section": record.section,
            "subsection": record.subsection,
            "position_in_document": record.position_in_document,
            "cross_id": record.cross_id,
            "value_str": record.value_str,
            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "deliting": record.deliting,
        }
        return res

    @staticmethod
    def try_delete_attribute(id):
        Document_attributes.objects.filter(id=id).delete()


class Spw_attrs(Document_attributes_view):
    objects = Spw_attrsManager()

    class Meta:
        verbose_name = 'Аттрибуты Спецификаций'
        proxy = True
        managed = False



# Generated by Django 2.2.7 on 2019-11-21 13:31

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0016_launch_item_timing_view'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='launch_item_timing_view',
            table='planing_launch_item_timing_view',
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-16 07:58

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0114_auto_20191215_2048'),
    ]

    operations = [
        migrations.AddField(
            model_name='lotsman_documents_hierarcy',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'), ('beenItemed', 'Был внесен в состав изделий')), db_index=True, default=1),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-17 16:23

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0117_auto_20191117_1622'),
    ]

    operations = [
        migrations.AlterField(
            model_name='statusdogovor',
            name='props',
            field=bitfield.models.BitField((('real', 'Реальный статус'), ('disabled', 'Неактивная запись в гриде')), db_index=True, default=1),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-08 12:53

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0152_remove_status_code_owner'),
        ('production', '0067_auto_20191008_0937'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operation_material',
            name='operation',
        ),
        migrations.RemoveField(
            model_name='operation_resources',
            name='operation',
        ),
        migrations.AddField(
            model_name='operation_resources',
            name='location',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='ckk.Locations'),
        ),
        migrations.AlterField(
            model_name='operation_resources',
            name='resource',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='production.Resource'),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-11 11:56

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0093_auto_20190611_1154'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='cdw_pdf_item_view',
            table='ckk_cdw_pdf_item_view',
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-10 12:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0207_auto_20200210_1248'),
    ]

    operations = [
        migrations.AddConstraint(
            model_name='materials',
            constraint=models.UniqueConstraint(condition=models.Q(('name', None), ('nomenklatura_model', None)), fields=('gost', 'location', 'materials_type'), name='Materials_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='materials',
            constraint=models.UniqueConstraint(condition=models.Q(nomenklatura_model=None), fields=('gost', 'location', 'materials_type', 'name'), name='Materials_unique_constraint_1'),
        ),
        migrations.AddConstraint(
            model_name='materials',
            constraint=models.UniqueConstraint(condition=models.Q(name=None), fields=('gost', 'location', 'materials_type', 'nomenklatura_model'), name='Materials_unique_constraint_2'),
        ),
        migrations.AddConstraint(
            model_name='materials',
            constraint=models.UniqueConstraint(fields=('gost', 'location', 'materials_type', 'name', 'nomenklatura_model'), name='Materials_unique_constraint_3'),
        ),
    ]



from django.urls import path

from kaf_pas.kd.views import uploades

urlpatterns = [

    path('Uploades/Fetch/', uploades.Uploades_Fetch),
    path('Uploades/Add', uploades.Uploades_Add),
    path('Uploades/Update', uploades.Uploades_Update),
    path('Uploades/Remove', uploades.Uploades_Remove),
    path('Uploades/Lookup/', uploades.Uploades_Lookup),
    path('Uploades/Info/', uploades.Uploades_Info),
    path('Uploades/Copy', uploades.Uploades_Copy),
    path('Uploades/Confirmation', uploades.Uploades_Confirmation),
    path('Uploades/UnConfirmation', uploades.Uploades_UnConfirmation),

]



import logging

from django.db.models import TextField, DecimalField, PositiveIntegerField, BooleanField, DateTimeField

from isc_common.auth.models.user import User
from isc_common.common import blinkString
from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyProtect, ForeignKeySetNull
from isc_common.models.base_ref import Hierarcy
from isc_common.models.standard_colors import Standard_colors
from isc_common.models.tree_audit import TreeAuditModelQuerySet, TreeAuditModelManager
from isc_common.number import DelProps
from kaf_pas.ckk.models.ed_izm import Ed_izm
from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.material_askon import Material_askon
from kaf_pas.ckk.models.materials import Materials
from kaf_pas.planing.models.levels import Levels
from kaf_pas.planing.models.operation_refs import Operation_refsManager
from kaf_pas.planing.models.operation_types import Operation_types
from kaf_pas.planing.models.operation_value import Operation_value
from kaf_pas.planing.models.status_operation_types import Status_operation_types
from kaf_pas.production.models.launches import Launches
from kaf_pas.production.models.operations import Operations
from kaf_pas.production.models.resource import Resource

logger = logging.getLogger(__name__)


class Operations_viewQuerySet(TreeAuditModelQuerySet):
    pass


class Operations_viewManager(TreeAuditModelManager):

    @staticmethod
    def getRecord(record):
        res = {
            'color__color': record.color.color if record.color else None,
            'color__name': record.color.name if record.color else None,
            'color_id': record.color.id if record.color else None,
            'creator__short_name': record.creator.get_short_name,
            'creator_id': record.creator.id,
            'date': record.date,
            'description': record.description,
            'edizm__code': record.value.edizm.code if record.value and record.value.edizm else None,
            'edizm__name': record.value.edizm.name if record.value and record.value.edizm else None,
            'edizm_id': record.value.edizm.id if record.value and record.value.edizm else None,
            'id': record.id,
            'isFolder': record.isFolder,
            'item__STMP_1__value_str': record.item.STMP_1.value_str if record.item and record.item.STMP_1 and record.item.STMP_1 else None,
            'item__STMP_1_id': record.item.STMP_1.id if record.item and record.item.STMP_1 and record.item.STMP_1 else None,
            'item__STMP_2__value_str': record.item.STMP_2.value_str if record.item and record.item.STMP_2 and record.item.STMP_2 else None,
            'item__STMP_2_id': record.item.STMP_2.id if record.item and record.item.STMP_2 and record.item.STMP_2 else None,
            'item_full_name': record.item_full_name,
            'item_id': record.item.id if record.item else None,
            'item_item_name': record.item.item_name if record.item else None,
            'launch_id': record.launch.id if record.launch else None,
            'location__code': record.resource.location.code if record.resource and record.resource else None,
            'location__full_name': record.resource.location.full_name if record.resource and record.resource else None,
            'location__name': record.resource.location.name if record.resource and record.resource else None,
            'location_id': record.resource.location.id if record.resource else None,
            'mark': record.mark,
            'num': record.num,
            'operation_level__code': record.operation_level.code if record.operation_level else None,
            'operation_level__name': record.operation_level.name if record.operation_level else None,
            'operation_level_id': record.operation_level.id if record.operation_level else None,
            'opertype__full_name': record.opertype.full_name,
            'opertype_id': record.opertype.id,
            'parent_id': record.parent_id,
            'prev_status_id': record.prev_status.id if record.prev_status else None,
            'production_operation__full_name': record.production_operation.full_name if record.production_operation else None,
            'production_operation__name': record.production_operation.name if record.production_operation else None,
            'production_operation_edizm__name': record.production_operation_edizm.name if record.production_operation_edizm else None,
            'production_operation_edizm_id': record.production_operation_edizm.id if record.production_operation_edizm else None,
            'production_operation_id': record.production_operation.id if record.production_operation else None,
            'production_operation_num': record.production_operation_num,
            'production_operation_qty': record.production_operation_qty,
            'resource__code': record.resource.code if record.resource and record.resource else None,
            'resource__description': record.resource.description if record.resource and record.resource else None,
            'resource__name': record.resource.name if record.resource and record.resource else None,
            'resource_id': record.resource.id if record.resource else None,
            'status__code': record.status.code if record.status else None,
            'status__name': blinkString(text=record.status.name, color=record.status.color, blink=record.status.props.blink) if record.status else None,
            'status_id': record.status.id if record.status else None,
            'value': record.value.value if record.value else None,
        }
        return DelProps(res)

    def get_queryset(self):
        return Operations_viewQuerySet(self.model, using=self._db)


class Operations_view(Hierarcy):
    color = ForeignKeySetNull(Standard_colors, null=True, blank=True)
    creator = ForeignKeyProtect(User, default=None)
    date = DateTimeField(default=None)
    description = TextField(null=True, blank=True)
    isFolder = BooleanField(default=None)
    item = ForeignKeySetNull(Item, null=True, blank=True)
    item_full_name = TextField(null=True, blank=True)
    launch = ForeignKeySetNull(Launches, null=True, blank=True)
    mark = CodeField()
    material = ForeignKeySetNull(Material_askon, null=True, blank=True)
    num = CodeField()
    operation_level = ForeignKeySetNull(Levels, null=True, blank=True)
    opertype = ForeignKeyProtect(Operation_types, default=None)
    prev_status = ForeignKeyProtect(Status_operation_types, related_name='planing_Operations_prev_status_view', null=True, blank=True)
    production_operation = ForeignKeySetNull(Operations, null=True, blank=True)
    production_operation_edizm = ForeignKeySetNull(Ed_izm, null=True, blank=True, related_name='Operations_view_ed_izm')
    production_operation_num = PositiveIntegerField(null=True, blank=True)
    production_operation_qty = DecimalField(decimal_places=4, max_digits=19, null=True, blank=True)
    props = Operation_refsManager.props()
    resource = ForeignKeySetNull(Resource, null=True, blank=True)
    status = ForeignKeyProtect(Status_operation_types, related_name='planing_Operations_status_view', null=True, blank=True)
    std_material = ForeignKeySetNull(Materials, null=True, blank=True)
    value = ForeignKeySetNull(Operation_value, null=True, blank=True, related_name='planing_Operations_prev_value_view')
    value1 = ForeignKeySetNull(Operation_value, null=True, blank=True, related_name='planing_Operations_prev_value1_view')

    objects = Operations_viewManager()

    def __str__(self):
        return f"ID:{self.id}, \n" \
               f"num:{self.num}, \n" \
               f"props: {self.props}, \n" \
               f"creator: [{self.creator}], \n" \
               f"date: {self.date}, \n" \
               f"description: {self.description}, \n" \
               f"opertype: [{self.opertype}], \n" \
               f"status: [{self.status}], \n" \
               f"isFolder: [{self.isFolder}], \n" \
               f"opertype: [{self.opertype}], \n" \
               f"item: [{self.item}], \n" \
               f"parent_id: {self.parent}, \n" \
               f"item_full_name: {self.item_full_name}, \n" \
               f"production_operation: [{self.production_operation}], \n" \
               f"launch: [{self.launch}], \n" \
               f"resource: [{self.resource}], \n" \
               f"operation_level: [{self.operation_level}], \n" \
               f"edizm: [{self.value.edizm}], \n" \
               f"material: [{self.material}], \n" \
               f"std_material: [{self.std_material}], \n" \
               f"value: {self.value.value}"

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Опреации системные'
        managed = False
        db_table = 'planing_operations_view'



# Generated by Django 3.0.3 on 2020-02-17 13:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0094_auto_20200217_1323'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operation_item',
            name='item_full_name',
            field=models.TextField(),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-08 14:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0202_auto_20200208_1406'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='locations_users',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='locations_users',
            constraint=models.UniqueConstraint(condition=models.Q(parent=None), fields=('location', 'user'), name='Locations_users_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='locations_users',
            constraint=models.UniqueConstraint(fields=('location', 'parent', 'user'), name='Locations_users_unique_constraint_1'),
        ),
    ]



import os
import shutil
import time
from tkinter import Tk
from tkinter.filedialog import askopenfilename

import timestring as timestring
from django.test import TestCase

from isc_common.auth.models.user import User
from kaf_pas.ckk.models.attr_type import Attr_type
from kaf_pas.common.constants import constants1, ColumnType
from kaf_pas.common.kompas_api5 import ksDocumentParam, ksDocument2D, ksSpcDocument, ksSpecification, ksSheetPar, ksStamp
from kaf_pas.common.kompas_constants import constants
from kaf_pas.kd.interaction.interaction_kompas_5 import InteractionKompas_5
from kaf_pas.kd.interaction.interaction_kompas_7 import InteractionKompas_7


class Test_Kompas(TestCase):

    def setUp(self):
        self.path = "path"
        self.user = User.objects.get(username='admin')

    # @skip
    def test_kompas5_открытие_Компаса_и_создание_нового_чертежа(self):
        interaction_kompas_3_d = InteractionKompas_5()
        kompas = interaction_kompas_3_d.kompasRun()

        kompas_7 = kompas.ksGetApplication7()
        kompas_7.HideMessage = constants.ksHideMessageNo

        print(kompas_7.ApplicationName(FullName=True))
        self.assertNotEquals(kompas, None)

        document_param = ksDocumentParam(kompas.GetParamStruct(constants.ko_DocumentParam))
        document_param.Init()
        document_param.type = constants.ksDocumentDrawing

        document2_d = ksDocument2D(kompas.Document2D())
        document2_d.ksCreateDocument(document_param)

        kompas.Visible = True

        time.sleep(5)
        kompas.Quit()
        time.sleep(5)

    # @skip
    def test_kompas5_открытие_Компаса_и_создание_новой_спецификации(self):
        interaction_kompas_3_d = InteractionKompas_5()
        kompas = interaction_kompas_3_d.kompasRun()
        kompas_7 = kompas.ksGetApplication7()
        kompas_7.HideMessage = constants.ksHideMessageNo

        print(kompas_7.ApplicationName(FullName=True))
        self.assertNotEquals(kompas, None)

        document_param = ksDocumentParam(kompas.GetParamStruct(constants.ko_DocumentParam))
        document_param.Init()
        document_param.type = 4  # lt_DocSpc
        document_param.regime = 0  # визуальный режим

        # Получаем  интерфейс параметров листа
        sheet_par = ksSheetPar(document_param.GetLayoutParam())
        sheet_par.Init()

        # Указываем библиотеку стилей
        ################################ По умолчанию принимается graphic.lyt #############################################################
        # str = kompas.ksSystemPath(0) + '\\graphic.lyt'
        # sheet_par.layoutName = str

        # Получаем интерфейс документа - спецификации
        spc_document = ksSpcDocument(kompas.SpcDocument())

        # Создаем спецификацию
        spc_document.ksCreateDocument(document_param)

        # Открываем существующую спецификацию
        kompas.Visible = True

        time.sleep(5)
        kompas.Quit()
        time.sleep(5)

    # @skip
    def test_kompas5_открытие_Компаса_и_наполнение_спецификации(self):
        interaction_kompas_3_d = InteractionKompas_5()
        kompas = interaction_kompas_3_d.kompasRun()
        kompas_7 = kompas.ksGetApplication7()
        kompas_7.HideMessage = constants.ksHideMessageNo

        print(kompas_7.ApplicationName(FullName=True))
        self.assertNotEquals(kompas, None)

        document_param = ksDocumentParam(kompas.GetParamStruct(constants.ko_DocumentParam))
        document_param.Init()
        document_param.type = 4  # lt_DocSpc
        document_param.regime = 0  # визуальный режим

        # Получаем  интерфейс параметров листа
        sheet_par = ksSheetPar(document_param.GetLayoutParam())
        sheet_par.Init()

        # Указываем библиотеку стилей
        # По умолчанию принимается graphic.lyt
        nameLib = kompas.ksSystemPath(0) + '\\graphic.lyt'
        sheet_par.layoutName = nameLib

        # Получаем интерфейс документа - спецификации
        spc_document = ksSpcDocument(kompas.SpcDocument())

        # Создаем спецификацию
        spc_document.ksCreateDocument(document_param)

        # Получаем интерфейс ksSpecification
        specification = ksSpecification(spc_document.GetSpecification())

        typeObj = 1
        # Cоздаем раздел "Документация"
        specification.ksSpcObjectCreate(nameLib=nameLib, styleNumb=1, secNumb=constants1.documentation, subSecNumb=0, numb=0, typeObj=typeObj)
        # Заполняем колонку "Наименование"
        specification.ksSetSpcObjectColumnText(ColumnType.SPC_CLM_NAME, 1, 0, 'Спецификация')
        # Заполняем колонку "Количество"
        specification.ksSpcCount(0, '1')
        # Закрываем объект в разделе "Документация"
        specification.ksSpcObjectEnd()

        # Создаем раздел "Детали"
        specification.ksSpcObjectCreate(nameLib=nameLib, styleNumb=1, secNumb=constants1.minutiae, subSecNumb=0, numb=0, typeObj=typeObj)
        # Заполняем колонку "Наименование"
        specification.ksSetSpcObjectColumnText(ColumnType.SPC_CLM_NAME, 1, 0, 'Крышка')
        # Заполняем колонку "Формат"
        specification.ksSetSpcObjectColumnText(ColumnType.SPC_CLM_FORMAT, 1, 0, 'А3')
        # Заполняем колонку "Количество"
        specification.ksSpcCount(0, '2')
        # Заполняем колонку "Позиция"
        specification.ksSpcPosition(1)
        # Заполняем колонку "Масса"
        specification.ksSpcMassa('25')
        # Закрываем объект в разделе "Детали"
        specification.ksSpcObjectEnd()

        # Открываем существующую спецификацию
        kompas.Visible = True

        time.sleep(5)
        kompas.Quit()
        time.sleep(5)

    # @skip
    def test_kompas5_открытие_Компаса_и_чтение_спецификации(self):
        interaction_kompas_3_d = InteractionKompas_5()
        kompas = interaction_kompas_3_d.kompasRun()
        kompas_7 = kompas.ksGetApplication7()
        kompas_7.HideMessage = constants.ksHideMessageNo

        print(kompas_7.ApplicationName(FullName=True))
        self.assertNotEquals(kompas, None)

        root = Tk()
        root.withdraw()  # Скрываем основное окно и сразу окно выбора файлов

        filename = askopenfilename(title="Выберите файл спецификации ", filetypes=[('Файлы спецификаций, "Компас 3D"', '*.spw'), ])

        def prnt(spec_row, sectionName, document):
            print(
                f"format: {spec_row.format} zone: {spec_row.zone} pos: {spec_row.pos} mark: {spec_row.mark} name: {spec_row.name} count: {spec_row.count} note: {spec_row.note} massa: {spec_row.massa} material: {spec_row.material} user: {spec_row.user} code: {spec_row.code} factory: {spec_row.factory}")

        # Получаем интерфейс документа - спецификации
        spc_document = ksSpcDocument(kompas.SpcDocument())
        # Открываем существующую спецификацию
        res_opening = spc_document.ksOpenDocument(filename, 0)

        if not res_opening:
            raise Exception(f'Документ: {filename} не открыт !!!!!!!')

        InteractionKompas_5.readSpecification(spc_document, kompas, prnt)

        kompas.Visible = True

        root.destroy()  # Уничтожаем основное окно
        root.mainloop()

    def print(self, text, index, document):
        code = f"STMP_{index}"
        attr = Attr_type.objects.get_attr(code=code)
        if attr:
            print(f"{attr.name}({index}): {text}")
        else:
            print(f"Unknown ({index}): {text}")

    # @skip
    def test_kompas5_открытие_Компаса_и_чтение_штампа_из_спецификации(self):
        interaction_kompas_3_d = InteractionKompas_5()
        kompas = interaction_kompas_3_d.kompasRun()
        kompas_7 = kompas.ksGetApplication7()
        kompas_7.HideMessage = constants.ksHideMessageNo

        print(kompas_7.ApplicationName(FullName=True))
        self.assertNotEquals(kompas, None)

        root = Tk()
        root.withdraw()  # Скрываем основное окно и сразу окно выбора файлов

        filename = askopenfilename(title="Выберите файл спецификации ", filetypes=[('Файлы спецификаций, "Компас 3D"', '*.spw'), ])
        # Получаем интерфейс документа - спецификации
        spc_document = ksSpcDocument(kompas.SpcDocument())
        # Открываем существующую спецификацию в "слепом" (без вывода на экран) варианте
        res_opening = spc_document.ksOpenDocument(filename, 1)
        if not res_opening:
            raise Exception(f'Документ: {filename} не открыт !!!!!!!')

        # Чтение штампа
        stamp = ksStamp(spc_document.GetStamp())
        InteractionKompas_5.readStampItem(kompas=kompas, stamp=stamp, function=self.print, document=None)
        stamp.ksCloseStamp()

        root.destroy()  # Уничтожаем основное окно
        root.mainloop()
        kompas.Quit()

    # @skip
    def test_kompas5_открытие_Компаса_и_чтение_штампа_из_чертежа(self):
        interaction_kompas_3_d = InteractionKompas_5()
        kompas = interaction_kompas_3_d.kompasRun()
        kompas_7 = kompas.ksGetApplication7()
        kompas_7.HideMessage = constants.ksHideMessageNo

        print(kompas_7.ApplicationName(FullName=True))
        self.assertNotEquals(kompas, None)

        root = Tk()
        root.withdraw()  # Скрываем основное окно и сразу окно выбора файлов

        filename = askopenfilename(title="Выберите файл чертежа ", filetypes=[('Файлы чертежей, "Компас 3D"', '*.cdw'), ])
        # Получаем интерфейс документа - чертежа
        document2_d = ksDocument2D(kompas.Document2D())
        # Открываем существующую спецификацию в "слепом" (без вывода на экран) варианте
        res_opening = document2_d.ksOpenDocument(filename, 1)
        if not res_opening:
            raise Exception(f'Документ: {filename} не открыт !!!!!!!')

        # Чтение штампа
        stamp = ksStamp(document2_d.GetStamp())

        InteractionKompas_5.readStampItem(kompas=kompas, stamp=stamp, function=self.print, document=None)
        stamp.ksCloseStamp()

        root.destroy()  # Уничтожаем основное окно
        root.mainloop()

        kompas.Quit()

    def test_kompas5_открытие_Компаса_и_сохранение_чертежа_в_растровом_формате(self):
        interaction_kompas_3_d = InteractionKompas_5()
        kompas = interaction_kompas_3_d.kompasRun()
        kompas_7 = kompas.ksGetApplication7()
        kompas_7.HideMessage = constants.ksHideMessageNo

        print(kompas_7.ApplicationName(FullName=True))
        self.assertNotEquals(kompas, None)

        root = Tk()
        root.withdraw()  # Скрываем основное окно и сразу окно выбора файлов

        filename = askopenfilename(title="Выберите файл чертежа ", filetypes=[('Файлы чертежей, "Компас 3D"', '*.cdw'), ])

        dir_tmp = f'c:{os.path.altsep}tmp_thumb'

        if os.path.exists(dir_tmp):
            shutil.rmtree(dir_tmp)
        os.mkdir(dir_tmp)

        # Получаем интерфейс чертежа
        document2_d = ksDocument2D(kompas.Document2D())
        # Открываем существующую спецификацию в "слепом" (без вывода на экран) варианте
        res_opening = document2_d.ksOpenDocument(filename, 1)
        if not res_opening:
            raise Exception(f'Документ: {filename} не открыт !!!!!!!')

        interactionKompas = InteractionKompas_5()
        files = interactionKompas.save_2_rastr(document=document2_d, filename=filename, scale=1.0, dir=dir_tmp)
        for file in files:
            print(file)
        root.destroy()  # Уничтожаем основное окно
        root.mainloop()

        kompas.Quit()

    def test_kompas5_открытие_Компаса_и_сохранение_спецификации_в_растровом_режиме(self):
        interaction_kompas_3_d = InteractionKompas_5()
        kompas = interaction_kompas_3_d.kompasRun()
        kompas_7 = kompas.ksGetApplication7()
        kompas_7.HideMessage = constants.ksHideMessageNo

        dir_tmp = f'c:{os.path.altsep}tmp_thumb'

        if os.path.exists(dir_tmp):
            shutil.rmtree(dir_tmp)
        os.mkdir(dir_tmp)

        print(kompas_7.ApplicationName(FullName=True))
        self.assertNotEquals(kompas, None)

        root = Tk()
        root.withdraw()  # Скрываем основное окно и сразу окно выбора файлов

        filename = askopenfilename(title="Выберите файл спецификации ", filetypes=[('Файлы спецификаций, "Компас 3D"', '*.spw'), ])

        # Получаем интерфейс документа - спецификации
        spc_document = ksSpcDocument(kompas.SpcDocument())
        # Открываем существующую спецификацию в "слепом" (без вывода на экран) варианте
        res_opening = spc_document.ksOpenDocument(filename, 1)
        if not res_opening:
            raise Exception(f'Документ: {filename} не открыт !!!!!!!')

        interactionKompas = InteractionKompas_5()
        files = interactionKompas.save_2_rastr(document=spc_document, filename=filename, scale=0.1, dir=dir_tmp)
        for file in files:
            print(file)

        root.destroy()  # Уничтожаем основное окно
        root.mainloop()

        kompas.Quit()

    # @skip
    def test_kompas7Run(self):
        interaction_kompas_3_d = InteractionKompas_7()
        res = interaction_kompas_3_d.kompasRun()
        self.assertNotEquals(res, None)
        res.Visible = True
        # res.Quit()
        # time.sleep(2)

    def test_str_2_date(self):
        print(timestring.Date('01/01/2018'))

# E:/KAF/Кузова/К5350/К5350-11-003/К5350-11-003(1174-1.3) Пенза Рубин 18-109/Надрамник/К5350-11-003(1174-1.3).00.00.100.spw



from kaf_pas.k_one_c.models_unmanaged.nomenklatura import Nomenklatura
from one_c.models.entities import Entities


class KafEntities(Entities):

    def get_field(self, id, name):
        _entity = [_entity for _entity in self.entities if _entity.id == id]
        if len(_entity) == 0:
            return None
        else:
            field = [field for field in _entity[0].used_field() if field.name == name]
            if len(field) == 0:
                return None
            else:
                return field[0]



# Generated by Django 3.0.3 on 2020-02-08 07:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0196_auto_20200208_0736'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='item',
            name='unique_with_optional',
        ),
        migrations.AddConstraint(
            model_name='item',
            constraint=models.UniqueConstraint(condition=models.Q(('STMP_1', None), ('STMP_2', None), ('version', None)), fields=('STMP_1', 'STMP_2', 'version', 'lotsman_type_id'), name='unique_with_optional'),
        ),
    ]



# Generated by Django 2.2 on 2019-04-14 04:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0010_auto_20190414_0429'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='documents_thumb10',
            managers=[
            ],
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-09 22:49

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0084_auto_20190608_1341'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'),), db_index=True, default=1),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-21 06:23

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0159_auto_20191119_2034'),
        ('planing', '0012_auto_20191121_0605'),
    ]

    operations = [
        migrations.RenameField(
            model_name='launch_item_refs_timing',
            old_name='qty',
            new_name='time_duration_operations',
        ),
        migrations.AddField(
            model_name='launch_item_refs_timing',
            name='ed_izm',
            field=isc_common.fields.related.ForeignKeyCascade(default=None, on_delete=django.db.models.deletion.CASCADE, to='ckk.Ed_izm'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-17 16:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0012_auto_20190517_1248'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='customer',
            options={'verbose_name': 'Клиент'},
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-09 10:32

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0130_remove_materials_ed_izm'),
        ('production', '0051_auto_20190616_0635'),
    ]

    operations = [
        migrations.AddField(
            model_name='operation_material',
            name='ed_izm',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Ed_izm'),
        ),
        migrations.AddField(
            model_name='operation_material',
            name='qty',
            field=models.DecimalField(decimal_places=4, default=0.0, max_digits=10),
        ),
        migrations.CreateModel(
            name='Operations_item',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('qty', models.PositiveIntegerField(default=None)),
                ('ed_izm', isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Ed_izm')),
                ('item', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Item')),
                ('operation', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='production.Operations')),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='production.Operations_item')),
            ],
            options={
                'verbose_name': 'Кросс таблица',
            },
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-02 08:40

from django.db import migrations, models
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0053_auto_20190731_2227'),
    ]

    operations = [
        migrations.CreateModel(
            name='Operations_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeField(blank=True, null=True)),
                ('name', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('isFolder', models.BooleanField()),
            ],
            options={
                'verbose_name': 'Операции',
                'db_table': 'production_operations_view',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Resource_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('code', isc_common.fields.code_field.CodeField(blank=True, null=True)),
                ('name', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('isFolder', models.BooleanField()),
            ],
            options={
                'verbose_name': 'Ресурсы',
                'db_table': 'production_resource_view',
                'managed': False,
            },
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-18 17:03

from django.db import migrations
import isc_common.fields.files


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0025_auto_20190517_2112'),
    ]

    operations = [
        migrations.AlterField(
            model_name='documents_thumb',
            name='attfile',
            field=isc_common.fields.files.FileFieldEx(blank=True, default=None, max_length=255, null=True, upload_to='', verbose_name='Файл'),
        ),
        migrations.AlterField(
            model_name='documents_thumb10',
            name='attfile',
            field=isc_common.fields.files.FileFieldEx(blank=True, default=None, max_length=255, null=True, upload_to='', verbose_name='Файл'),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-04 09:19

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0017_auto_20191121_1331'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_item_refs_timing',
            name='ed_izm',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Ed_izm'),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-17 09:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0091_auto_20200217_0905'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='operation_level',
            options={'verbose_name': 'Кросс-таблица'},
        ),
        migrations.AlterField(
            model_name='levels',
            name='code',
            field=models.PositiveIntegerField(default=0),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-17 12:48

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0010_demand'),
    ]

    operations = [
        migrations.AddField(
            model_name='demand',
            name='precent',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='sales.Precent'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-16 15:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0006_auto_20190516_1548'),
    ]

    operations = [
        migrations.AddField(
            model_name='customer',
            name='description',
            field=models.TextField(blank=True, default=None, null=True, verbose_name='Описание'),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-08 07:31

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0152_remove_status_code_owner'),
        ('production', '0065_operations_item_num'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='operations_item',
            unique_together={('item', 'operation', 'num')},
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-18 14:36

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0059_auto_20200117_1859'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operation_types',
            name='props',
            field=bitfield.models.BitField((('plus', 'Приходная операция'), ('minus', 'Расходная операция')), db_index=True, default=0),
        ),
    ]



# Generated by Django 2.2 on 2019-04-14 08:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0014_auto_20190414_0801'),
    ]

    operations = [
        migrations.AddField(
            model_name='document_attrs',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AddField(
            model_name='documents',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AddField(
            model_name='documents_thumb',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AddField(
            model_name='documents_thumb10',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AddField(
            model_name='pathes',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-10 18:32

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0088_auto_20190610_1000'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='item_line',
            unique_together={('parent', 'child')},
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-17 09:05

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0089_levels'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operation_level',
            name='level',
            field=isc_common.fields.related.ForeignKeyCascade(default=None, on_delete=django.db.models.deletion.CASCADE, related_name='planing_lev_1', to='planing.Levels'),
        ),
        migrations.AlterField(
            model_name='operation_level',
            name='operation',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, related_name='planing_operation_1', to='planing.Operations'),
        ),
    ]



from django.urls import path

from kaf_pas.kd.views import document_attributes_STMP_120

urlpatterns = [

    path('Document_attributes_STMP_120/Fetch/', document_attributes_STMP_120.Document_attributes_STMP_120_Fetch),
    path('Document_attributes_STMP_120/Add', document_attributes_STMP_120.Document_attributes_STMP_120_Add),
    path('Document_attributes_STMP_120/Update', document_attributes_STMP_120.Document_attributes_STMP_120_Update),
    path('Document_attributes_STMP_120/Remove', document_attributes_STMP_120.Document_attributes_STMP_120_Remove),
    path('Document_attributes_STMP_120/Lookup/', document_attributes_STMP_120.Document_attributes_STMP_120_Lookup),
    path('Document_attributes_STMP_120/Info/', document_attributes_STMP_120.Document_attributes_STMP_120_Info),
    path('Document_attributes_STMP_120/Copy', document_attributes_STMP_120.Document_attributes_STMP_120_Copy),

]



from django.urls import path

from kaf_pas.sales.views import precent_item_types

urlpatterns = [

    path('Precent_item_types/Fetch/', precent_item_types.Precent_item_types_Fetch),
    path('Precent_item_types/Add', precent_item_types.Precent_item_types_Add),
    path('Precent_item_types/Update', precent_item_types.Precent_item_types_Update),
    path('Precent_item_types/Remove', precent_item_types.Precent_item_types_Remove),
    path('Precent_item_types/Lookup/', precent_item_types.Precent_item_types_Lookup),
    path('Precent_item_types/Info/', precent_item_types.Precent_item_types_Info),
    path('Precent_item_types/Copy', precent_item_types.Precent_item_types_Copy),

]



# Generated by Django 3.0.3 on 2020-02-09 12:44

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0035_auditmodelex'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('ckk', '0206_auto_20200209_1230'),
        ('production', '0176_auto_20200209_1207'),
        ('planing', '0086_auto_20200208_1424'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='operation_color',
            unique_together={('operation', 'color')},
        ),
        migrations.AlterUniqueTogether(
            name='operation_executor',
            unique_together={('operation', 'executor')},
        ),
        migrations.AlterUniqueTogether(
            name='operation_item',
            unique_together={('operation', 'item')},
        ),
        migrations.AlterUniqueTogether(
            name='operation_launches',
            unique_together={('operation', 'launch')},
        ),
        migrations.AlterUniqueTogether(
            name='operation_level',
            unique_together={('operation', 'level')},
        ),
        migrations.AlterUniqueTogether(
            name='operation_locations',
            unique_together={('operation', 'location')},
        ),
        migrations.AlterUniqueTogether(
            name='operation_material',
            unique_together={('operation', 'material')},
        ),
        migrations.AlterUniqueTogether(
            name='operation_resources',
            unique_together={('operation', 'resource')},
        ),
        migrations.AlterUniqueTogether(
            name='operation_standard_prod',
            unique_together={('operation', 'material')},
        ),
        migrations.AlterUniqueTogether(
            name='operation_types_users',
            unique_together={('operation_type', 'user')},
        ),
    ]



from django.urls import path

from kaf_pas.system.views import contants

urlpatterns = [

    path('Contants/Fetch/', contants.Contants_Fetch),
    path('Contants/Add', contants.Contants_Add),
    path('Contants/Update', contants.Contants_Update),
    path('Contants/RefreshMatView', contants.Contants_RefreshMatView),
    path('Contants/Remove', contants.Contants_Remove),
    path('Contants/fixed_num_in_operations', contants.Contants_fixed_num_in_operations),
    path('Contants/Lookup/', contants.Contants_Lookup),
    path('Contants/Info/', contants.Contants_Info),
    path('Contants/Copy', contants.Contants_Copy),

]



# Generated by Django 2.2.1 on 2019-05-29 13:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0036_operation_material'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='operation_resources',
            name='deleted_at',
        ),
        migrations.RemoveField(
            model_name='operation_resources',
            name='deliting',
        ),
        migrations.RemoveField(
            model_name='operation_resources',
            name='editing',
        ),
        migrations.RemoveField(
            model_name='operation_resources',
            name='id_old',
        ),
        migrations.RemoveField(
            model_name='operation_resources',
            name='lastmodified',
        ),
        migrations.AlterField(
            model_name='operation_resources',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-24 14:48

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0111_auto_20200224_0413'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operation_refs',
            name='props',
            field=bitfield.models.BitField((('inner_routing', 'Связь операций внутри товарной позиции'), ('outer_routing', 'Связь операций между товарными позициями'), ('product_order_routing', 'Связи в блоке заказы на производство')), db_index=True, default=0),
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-08 12:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0138_material_type_view'),
    ]

    operations = [
        migrations.AddField(
            model_name='item',
            name='version',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-29 18:42

import django.db.models.deletion
from django.db import migrations

import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0107_auto_20191029_1437'),
    ]

    operations = [
        migrations.AlterField(
            model_name='precent',
            name='precent_item_type',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='sales.Precent_item_types'),
        ),
        migrations.AlterField(
            model_name='precent',
            name='precent_type',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='sales.Precent_types'),
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-20 07:33

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0135_auto_20190719_1925'),
        ('kd', '0067_auto_20190720_0729'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='document_attributes',
            unique_together={('value_str', 'attr_type')},
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-21 20:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('accounting', '0006_tmp_buffer_description'),
    ]

    operations = [
        migrations.AddField(
            model_name='tmp_buffer',
            name='value_off',
            field=models.DecimalField(blank=True, decimal_places=4, max_digits=19, null=True),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-17 13:56

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0092_launch_operation_resources'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Launch_item_refs',
        ),
    ]



import logging

from bitfield import BitField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditModel, AuditQuerySet, AuditManager

from kaf_pas.kd.models.document_attributes import Document_attributes
from kaf_pas.kd.models.documents import Documents

logger = logging.getLogger(__name__)


class Item_Complex_viewQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Item_Complex_viewManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            "id": record.id,
            "STMP_1_id": record.STMP_1.id if record.STMP_1 else None,
            "STMP_1__value_str": record.STMP_1.value_str if record.STMP_1 else None,
            "STMP_2_id": record.STMP_2.id if record.STMP_2 else None,
            "STMP_2__value_str": record.STMP_2.value_str if record.STMP_2 else None,
            "lastmodified": record.lastmodified,
            "document_id": record.document.id if record.document else None,
            "document__file_document": record.document.file_document if record.document else None,
            "editing": record.editing,
            "deliting": record.deliting,
        }
        # print(res)
        return res

    def get_queryset(self):
        return Item_Complex_viewQuerySet(self.model, using=self._db)


class Item_Complex_view(AuditModel):
    STMP_1 = ForeignKeyProtect(Document_attributes, verbose_name='Наименование изделия', related_name='STMP_1_complex', null=True, blank=True)
    STMP_2 = ForeignKeyProtect(Document_attributes, verbose_name='Обозначение изделия', related_name='STMP_2_complex', null=True, blank=True)
    document = ForeignKeyProtect(Documents, verbose_name='Документ', null=True, blank=True)
    props = BitField(flags=(
        ('relevant', 'Актуальность'),
        ('from_cdw', 'Получено из чертежа'),
        ('from_spw', 'Получено из спецификации'),
        ('for_line', 'Строка спецификации'),
        ('from_pdf', 'Получено из бумажного архива'),
    ), default=1, db_index=True)

    objects = Item_Complex_viewManager()

    @property
    def item(self):
        from kaf_pas.ckk.models.item import Item
        return Item.objects.get(id=self.id)

    def __str__(self):
        return f"ID={self.id} STMP_1=[{self.STMP_1}], STMP_2=[{self.STMP_2}], props={self.props}"


    class Meta:
        managed = False
        db_table = 'ckk_complex_item_view'
        verbose_name = 'Сборочные еденицы'



# Generated by Django 2.2.1 on 2019-05-28 17:09

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0040_precent_locations'),
    ]

    operations = [
        migrations.CreateModel(
            name='Precent_items_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('qty', models.PositiveIntegerField()),
            ],
            options={
                'verbose_name': 'Комплектация распоряжения',
                'db_table': 'sales_precent_items_view',
                'managed': False,
            },
        ),
        migrations.AlterField(
            model_name='precent',
            name='parent',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='sales.Precent'),
        ),
    ]



import logging

from django.db.models import Model, QuerySet, Manager, UniqueConstraint, Q, CheckConstraint, F

from isc_common.fields.related import ForeignKeyCascade, ForeignKeyProtect
from kaf_pas.kd.models.document_attributes import Document_attributes
from kaf_pas.kd.models.lotsman_documents_hierarcy import Lotsman_documents_hierarcy

logger = logging.getLogger(__name__)


class Lotsman_document_attr_cross1QuerySet(QuerySet):
    def delete(self):
        return super().delete()

    def create(self, **kwargs):
        return super().create(**kwargs)

    def update(self, **kwargs):
        return super().create(**kwargs)

    def update_or_create(self, defaults=None, **kwargs):
        return super().update_or_create(**kwargs, defaults=defaults)

    def get_or_create(self, defaults=None, **kwargs):
        return super().get_or_create(**kwargs, defaults=defaults)

    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Lotsman_document_attr_cross1Manager(Manager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Lotsman_document_attr_cross1QuerySet(self.model, using=self._db)


class Lotsman_document_attr_cross(Model):
    document = ForeignKeyCascade(Lotsman_documents_hierarcy, related_name='Lotsman_document_attr_cross_doc')
    parent_document = ForeignKeyCascade(Lotsman_documents_hierarcy, related_name='Lotsman_document_attr_cross_parent_doc', null=True, blank=True)
    attribute = ForeignKeyProtect(Document_attributes)

    objects = Lotsman_document_attr_cross1Manager()

    def __str__(self):
        return f"{self.id}, document: [{self.document}], attribute: [{self.attribute}]"

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Кросс таблица'
        constraints = [
            CheckConstraint(check=~Q(document=F('parent_document')), name='Lotsman_document_attr_cross_not_circulate_refs'),
            UniqueConstraint(fields=['attribute', 'document'], condition=Q(parent_document=None), name='Lotsman_document_attr_cross_unique_constraint_0'),
            UniqueConstraint(fields=['attribute', 'document', 'parent_document'], name='Lotsman_document_attr_cross_unique_constraint_1'),
        ]




# Generated by Django 3.0.3 on 2020-02-19 05:00

import bitfield.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0100_delete_operation_locations'),
    ]

    operations = [
        migrations.RemoveConstraint(
            model_name='operation_refs',
            name='Operation_refs_unique_constraint_0',
        ),
        migrations.RemoveConstraint(
            model_name='operation_refs',
            name='Operation_refs_unique_constraint_1',
        ),
        migrations.AddField(
            model_name='operation_refs',
            name='props',
            field=bitfield.models.BitField((('inner_routing', 'Связь операций внутри товарной позиции'), ('outer_routing', 'Связь операций между товарными позициями')), db_index=True, default=0),
        ),
        migrations.AlterUniqueTogether(
            name='operation_refs',
            unique_together={('child', 'parent', 'props')},
        ),
        migrations.AddConstraint(
            model_name='operation_refs',
            constraint=models.UniqueConstraint(condition=models.Q(parent=None), fields=('child', 'props'), name='Operation_refs_unique_constraint_0'),
        ),
        migrations.AddConstraint(
            model_name='operation_refs',
            constraint=models.UniqueConstraint(fields=('child', 'props', 'parent'), name='Operation_refs_unique_constraint_1'),
        ),
    ]



# Generated by Django 3.0.2 on 2020-01-20 10:28

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0185_auto_20200117_1305'),
        ('production', '0167_auto_20200120_1025'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operations_item',
            name='item',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='ckk.Item'),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-20 19:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0008_prod_tasks'),
    ]

    operations = [
        migrations.AlterField(
            model_name='prod_tasks',
            name='create_date',
            field=models.DateTimeField(verbose_name='Дата'),
        ),
    ]



from django.urls import path

from kaf_pas.planing.views import posting_items

urlpatterns = [

    path('Posting_items/Fetch/', posting_items.Posting_items_Fetch),
    path('Posting_items/Add', posting_items.Posting_items_Add),
    path('Posting_items/Update', posting_items.Posting_items_Update),
    path('Posting_items/Remove', posting_items.Posting_items_Remove),
    path('Posting_items/Lookup/', posting_items.Posting_items_Lookup),
    path('Posting_items/Info/', posting_items.Posting_items_Info),
    path('Posting_items/Copy', posting_items.Posting_items_Copy),

]



# Generated by Django 2.2.8 on 2019-12-13 12:52

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0177_item_refs_hierarcy'),
        ('kd', '0087_auto_20191213_1243'),
    ]

    operations = [
        migrations.AddField(
            model_name='lotsman_documents',
            name='attr_type',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='ckk.Attr_type', verbose_name='Тип атрибута'),
        ),
        migrations.AddField(
            model_name='lotsman_documents',
            name='value_int',
            field=models.BigIntegerField(blank=True, db_index=True, null=True, verbose_name='Значение атрибута'),
        ),
        migrations.AddField(
            model_name='lotsman_documents',
            name='value_str',
            field=models.TextField(blank=True, db_index=True, null=True, verbose_name='Значение атрибута'),
        ),
    ]



from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ('ckk', '0042_auto_20190426_1017'),
    ]
    atomic = True

    operations = [
        # migrations.RunSQL(
        #     f"""insert into ckk_status(lastmodified , editing, deliting, code, name, description )
        #                             values('NOW()', 'false' , 'false' ,'new', 'Новый', null )"""
        # ),

    ]



import logging

from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditQuerySet, AuditManager, AuditModel

from kaf_pas.ckk.models.item import Item
from kaf_pas.ckk.models.locations import Locations

logger = logging.getLogger(__name__)


class Item_locationQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Item_locationManager(AuditManager):

    @staticmethod
    def getRecord(record):
        res = {
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Item_locationQuerySet(self.model, using=self._db)


class Item_location(AuditModel):
    item = ForeignKeyProtect(Item)
    location = ForeignKeyProtect(Locations)

    objects = Item_locationManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        unique_together = (('item', 'location'),)
        verbose_name = 'Размещение товарных позиций по местам'



from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse
from kaf_pas.production.models.ready_2_launch_detail import Ready_2_launch_detail, Ready_2_launch_detailManager


@JsonResponseWithException()
def Ready_2_launch_detail_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Ready_2_launch_detail.objects.
                filter().
                get_range_rows1(
                request=request,
                function=Ready_2_launch_detailManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ready_2_launch_detail_Add(request):
    return JsonResponse(DSResponseAdd(data=Ready_2_launch_detail.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ready_2_launch_detail_Update(request):
    return JsonResponse(DSResponseUpdate(data=Ready_2_launch_detail.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ready_2_launch_detail_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Ready_2_launch_detail.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ready_2_launch_detail_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Ready_2_launch_detail.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ready_2_launch_detail_Info(request):
    return JsonResponse(DSResponse(request=request, data=Ready_2_launch_detail.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Ready_2_launch_detail_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Ready_2_launch_detail.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



# Generated by Django 3.0.2 on 2020-01-22 06:01

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0079_auto_20200121_2231'),
    ]

    operations = [
        migrations.CreateModel(
            name='Operation_color',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('color', isc_common.fields.code_field.ColorField()),
                ('operation', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='planing.Operations')),
            ],
            options={
                'verbose_name': 'Кросс таблица значений операции',
            },
        ),
    ]



# Generated by Django 2.2 on 2019-04-25 17:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0002_auto_20190425_1705'),
    ]

    operations = [
        migrations.AddField(
            model_name='operation_plan_template_locations',
            name='distribution_coefficient',
            field=models.IntegerField(default=1, verbose_name='коэффициент распределения'),
        ),
    ]



from django.urls import path

from kaf_pas.kd.views import spw_attrs

urlpatterns = [

    path('Spw_attrs/Fetch/', spw_attrs.Spw_attrs_Fetch),
    path('Spw_attrs/Info/', spw_attrs.Spw_attrs_Info),
    path('Spw_attrs/Update', spw_attrs.Spw_attrs_Update),
    # path('Spw_attrs/Remove', spw_attrs.Spw_attrs_Remove),
]



# Generated by Django 2.2 on 2019-04-17 15:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0019_auto_20190415_1307'),
    ]

    operations = [
        migrations.CreateModel(
            name='Cdw_attrs',
            fields=[
            ],
            options={
                'verbose_name': 'Чертеж',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('kd.document_attrs',),
        ),
        migrations.CreateModel(
            name='Spw_attrs',
            fields=[
            ],
            options={
                'verbose_name': 'Аттрибуты Спецификаций',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('kd.document_attrs',),
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-31 08:56

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0083_ready_2_launch_props'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ready_2_launch',
            name='props',
            field=bitfield.models.BitField((('check_qty', 'Проверять длительность'), ('check_num', 'Проверять № п/п'), ('check_material', 'Проверять материалы'), ('check_resources', 'Проверять ресурсы'), ('check_edizm', 'Проверять еденицу измерения'), ('check_operation', 'Проверять операцию')), db_index=True, default=0),
        ),
    ]



import logging

from django.db import transaction

from isc_common import setAttr, delAttr
from isc_common.fields.related import ForeignKeyCascade
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from kaf_pas.ckk.models.attr_type import Attr_type
from kaf_pas.production.models.operations import Operations

logger = logging.getLogger(__name__)


class Operation_attrQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def create(self, **kwargs):
        operation_id = kwargs.get('operation_id')

        res = None
        with transaction.atomic():
            if isinstance(operation_id, list):
                for op in operation_id:
                    setAttr(kwargs, 'operation_id', op)
                    delAttr(kwargs, 'attr_type__full_name')
                    delAttr(kwargs, 'attr_type__code')
                    delAttr(kwargs, 'attr_type__description')

                    res = super().create(**kwargs)
        return res


class Operation_attrManager(CommonManagetWithLookUpFieldsManager):

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            "attr_type_id": record.attr_type.id,
            "attr_type__code": record.attr_type.code,
            "attr_type__name": record.attr_type.name,
            "attr_type__full_name": record.attr_type.full_name,
            "attr_type__description": record.attr_type.description,
            'editing': record.editing,
            'deliting': record.deliting,
        }
        return res

    def get_queryset(self):
        return Operation_attrQuerySet(self.model, using=self._db)


class Operation_attr(AuditModel):
    operation = ForeignKeyCascade(Operations)
    attr_type = ForeignKeyCascade(Attr_type)

    objects = Operation_attrManager()

    def __str__(self):
        return f"ID:{self.id}, attr: [{self.attr_type}] , operation: [{self.operation}]"

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Кросс-таблица'
        unique_together = (('operation', 'attr_type'),)



# Generated by Django 2.2.2 on 2019-06-07 19:23

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0082_auto_20190607_1922'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_line',
            name='STMP_1',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_NAME', to='kd.Document_attributes', verbose_name='Наименование'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='STMP_2',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='SPC_CLM_MARK', to='kd.Document_attributes', verbose_name='Обозначение'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-21 18:53

import bitfield.models
from django.db import migrations
import django.db.models.deletion
import isc_common.fields.code_field
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0043_auto_20190516_1628'),
    ]

    operations = [
        migrations.AlterField(
            model_name='attr_type',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'),), db_index=True, default=1),
        ),
        migrations.AlterField(
            model_name='item',
            name='STMP_1',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='STMP_1', to='kd.Document_attrs', verbose_name='Наименование изделия'),
        ),
        migrations.AlterField(
            model_name='item',
            name='STMP_2',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='STMP_2', to='kd.Document_attrs', verbose_name='Обозначение изделия'),
        ),
        migrations.AlterField(
            model_name='item',
            name='document',
            field=isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='kd.Documents', verbose_name='Документ'),
        ),
        migrations.AlterField(
            model_name='item',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'), ('from_cdw', 'Получено из чертежа'), ('from_spw', 'Получено из спецификации'), ('for_line', 'Строка спецификации'), ('from_pdf', 'Получено из бумажного архива')), db_index=True, default=1),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='child',
            field=isc_common.fields.related.ForeignKeyCascade(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='item_child', to='ckk.Item', verbose_name='Товарная позиция'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='parent',
            field=isc_common.fields.related.ForeignKeyCascade(default=None, on_delete=django.db.models.deletion.CASCADE, related_name='item_parent', to='ckk.Item', verbose_name='Товарная позиция'),
        ),
        migrations.AlterField(
            model_name='item_line',
            name='props',
            field=bitfield.models.BitField((('relevant', 'Актуальность'),), db_index=True, default=1),
        ),
        migrations.AlterField(
            model_name='item_refs',
            name='child',
            field=isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, related_name='child', to='ckk.Item'),
        ),
        migrations.AlterField(
            model_name='item_refs',
            name='parent',
            field=isc_common.fields.related.ForeignKeyCascade(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='parent', to='ckk.Item'),
        ),
        migrations.AlterField(
            model_name='status',
            name='code',
            field=isc_common.fields.code_field.CodeField(),
        ),
        migrations.AlterUniqueTogether(
            name='status',
            unique_together={('code_owner', 'code')},
        ),
    ]



# Generated by Django 3.0.5 on 2020-04-13 14:54

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0221_item_qty'),
    ]

    operations = [
        migrations.AlterField(
            model_name='item_qty',
            name='props',
            field=bitfield.models.BitField((('items', 'Состав изделия (Технологическая спецификация)'), ('teach', 'Производственная спецификация')), db_index=True, default=1),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-14 15:37

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0103_auto_20191214_1434'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document_attr_cross',
            name='attribute',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='kd.Document_attributes'),
        ),
        migrations.AlterField(
            model_name='lotsman_document_attr_cross',
            name='attribute',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='kd.Document_attributes'),
        ),
    ]



import logging

from bitfield import BitField
from django.conf import settings
from django.db.models import TextField, PositiveIntegerField, DateTimeField, BigAutoField, BooleanField, Model
from django.utils import timezone

from isc_common import delAttr, setAttr
from isc_common.fields.code_field import CodeField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from kaf_pas.ckk.models.attr_type import Attr_type
from kaf_pas.kd.models.document_attributes import Document_attributes
from kaf_pas.kd.models.documents import DocumentManager, Documents, DocumentQuerySet
from kaf_pas.kd.models.pathes import Pathes

logger = logging.getLogger(__name__)


class Document_viewQuerySet(DocumentQuerySet):
    def _append_item_documents(self, request):
        from kaf_pas.ckk.models.item_document import Item_document

        # settings.LOCKS.acquire('Document_viewQuerySet._append_item_documents')
        item_id = request.json.get('data').get('item_id')
        if item_id != None:
            id = request.json.get('data').get('id')
            delAttr(request.json.get('data'), 'item_id')
            document_ids = [item.document.id for item in Item_document.objects.filter(item_id=item_id)]
            document_ids.append(id)
            setAttr(request.json.get('data'), 'id', document_ids)
        # settings.LOCKS.release('Document_viewQuerySet._append_item_documents')
        return request

    def get_info(self, request, *args):
        request = DSRequest(request=request)
        request=self._append_item_documents(request=request)
        criteria = self.get_criteria(json=request.json)
        cnt = super().filter(*args, criteria).count()
        cnt_all = super().filter().count()
        return dict(qty_rows=cnt, all_rows=cnt_all)

    def get_range_rows1(self, request, function=None, distinct_field_names=None, remove_fields=None):
        request = DSRequest(request=request)
        request = self._append_item_documents(request=request)
        self.alive_only = request.alive_only
        self.enabledAll = request.enabledAll
        res = self.get_range_rows(start=request.startRow, end=request.endRow, function=function, distinct_field_names=distinct_field_names, json=request.json, criteria=request.get_criteria())
        return res


class Documents_viewManager(DocumentManager):
    def get_queryset(self):
        return Document_viewQuerySet(self.model, using=self._db)

    def deleteFromRequest(self, request):
        _request = DSRequest(request=request)
        if _request.json:
            ids = _request.get_ids()
            old_ids = _request.get_old_ids()
        else:
            ids = dict(request.GET).get('ids')
            old_ids = []

        Documents.objects.filter(id__in=ids).delete()
        Documents.objects.filter(id__in=old_ids).delete()
        return []


class Documents_view(Model):
    id = BigAutoField(primary_key=True, verbose_name="Идентификатор")
    deleted_at = DateTimeField(verbose_name="Дата мягкого удаления", null=True, blank=True, db_index=True)
    editing = BooleanField(verbose_name="Возможность редактирования", default=True)
    deliting = BooleanField(verbose_name="Возможность удаления", default=True)
    lastmodified = DateTimeField(verbose_name='Последнее обновление', editable=False, db_index=True, default=timezone.now)

    path = ForeignKeyProtect(Pathes, verbose_name='Путь к документу')
    relevant = CodeField(verbose_name='Актуальность')
    attr_type = ForeignKeyProtect(Attr_type, verbose_name='Тип документа')
    file_document = TextField(verbose_name='Полный путь к файлу')
    file_size = PositiveIntegerField(verbose_name='Размер файла')
    file_modification_time = DateTimeField(verbose_name='Дата время поcледнего модификации документа', null=True, blank=True)
    file_access_time = DateTimeField(verbose_name='Дата время поcледнего доступа к документу', null=True, blank=True)
    file_change_time = DateTimeField(verbose_name='Дата время изменнения документа', null=True, blank=True)
    props = BitField(flags=(('relevant', 'Актуальность'), ('beenItemed', 'Был внесен в состав изделий')), default=1)
    load_error = TextField(null=True, blank=True)

    STMP_1 = ForeignKeyProtect(Document_attributes, verbose_name='Наименование изделия', related_name='STMP_1_doc_view', null=True, blank=True)
    STMP_2 = ForeignKeyProtect(Document_attributes, verbose_name='Обозначение изделия', related_name='STMP_2_doc_view', null=True, blank=True)

    @staticmethod
    def getRecord(record):
        res = {
            'id': record.id,
            'path_id': record.path.id,
            'attr_type_id': record.attr_type.id,
            'attr_type__code': record.attr_type.code,
            'attr_type__name': record.attr_type.name,
            "STMP_1_id": record.STMP_1.id if record.STMP_1 else None,
            "STMP_1__value_str": record.STMP_1.value_str if record.STMP_1 else None,
            "STMP_2_id": record.STMP_2.id if record.STMP_2 else None,
            "STMP_2__value_str": record.STMP_2.value_str if record.STMP_2 else None,
            'file_document': record.file_document,
            'file_size': record.file_size,
            'file_modification_time': record.file_modification_time,
            'file_access_time': record.file_access_time,
            'file_change_time': record.file_change_time,
            'relevant': record.relevant,
            'load_error': f'<pre>{record.load_error}</pre>' if record.load_error else None,
            'lastmodified': record.lastmodified,
            'editing': record.editing,
            'deliting': record.deliting,
            'deleted_at': record.deleted_at,
        }
        return res

    def __str__(self):
        return f'ID: {self.id}, STMP_1: {self.STMP_1}, STMP_2: {self.STMP_2} file: {self.file_document}'

    objects = Documents_viewManager()

    class Meta:
        managed = False
        db_table = 'kd_documents_view'
        verbose_name = 'Documents View'



# Generated by Django 2.2.5 on 2019-09-18 14:37

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sales', '0055_auto_20190918_1437'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='precent',
            name='customer',
        ),
        migrations.RemoveField(
            model_name='precent',
            name='type',
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-04 21:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0124_auto_20190703_2206'),
    ]

    operations = [
        migrations.AddField(
            model_name='material_params',
            name='order',
            field=models.PositiveIntegerField(default=0),
        ),
    ]



# Generated by Django 2.2.8 on 2019-12-06 11:51

from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('planing', '0024_auto_20191206_1151'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operations',
            name='opertype',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='planing.Operation_types'),
        ),
        migrations.AlterField(
            model_name='operations',
            name='value',
            field=models.DecimalField(decimal_places=4, max_digits=19),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-17 16:41

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0015_auto_20190426_1401'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operationsplan',
            name='startdate',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, verbose_name='Дата начала'),
        ),
        migrations.DeleteModel(
            name='Temp_production_operationsplan',
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-29 12:56

from django.db import migrations
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0052_pathes_props'),
    ]

    operations = [
        migrations.AlterField(
            model_name='documents_thumb',
            name='document',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='kd.Documents', verbose_name='КД'),
        ),
        migrations.AlterField(
            model_name='documents_thumb10',
            name='document',
            field=isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='kd.Documents', verbose_name='КД'),
        ),
    ]



# Generated by Django 2.2 on 2019-04-19 00:23

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('kd', '0021_documents_view'),
        ('ckk', '0032_item_location'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='item_image_refs',
            unique_together={('item', 'thumb10'), ('item', 'thumb')},
        ),
    ]



# Generated by Django 2.2.6 on 2019-10-08 09:37

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('ckk', '0152_remove_status_code_owner'),
        ('production', '0066_auto_20191008_0731'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='operations_item',
            unique_together={('item', 'num')},
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-19 14:50

import bitfield.models
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('production', '0121_auto_20191119_1418'),
    ]

    operations = [
        migrations.AlterField(
            model_name='launch_operations_material',
            name='props',
            field=bitfield.models.BitField((('enabled', 'Доступность в данной производственной спецификации'), ('added', 'Добавлено в данную производственную спецификацию'), ('edited', 'Изменено в данной производственной спецификации'), ('refreshed', 'Удалено из данной производственной спецификации')), db_index=True, default=1),
        ),
    ]
